#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

"""
    ProjectVariables
    ================
     Manage creation of CMake variables that will be used during compilation
"""

import os

from cmake_converter.utils import write_property_of_settings, is_settings_has_data
from cmake_converter.utils import write_comment, replace_vs_vars_with_cmake_vars
from cmake_converter.utils import cleaning_output, message, check_for_relative_in_path
from cmake_converter.utils import set_native_slash, get_dir_name_with_vars


class ProjectVariables:
    """
        Class who manage project variables
    """

    @staticmethod
    def set_target_name(context, target_name_value):
        context.settings[context.current_setting]['TARGET_NAME'] = [replace_vs_vars_with_cmake_vars(
            context,
            target_name_value
        )]
        message(context, 'TargetName = {}'.format(target_name_value), '')

    @staticmethod
    def set_output_dir_impl(context, output_node_text):
        """

        :param context:
        :param output_node_text:
        :return:
        """
        output_path = cleaning_output(context, output_node_text)
        output_path = output_path.strip().replace('\n', '')
        output_path = check_for_relative_in_path(context, output_path)
        context.settings[context.current_setting]['OUTPUT_DIRECTORY'] = [output_path]
        message(context, 'Output Dir = {0}'.format(output_path), '')

    @staticmethod
    def set_output_file_impl(context, output_file_node_text):
        if output_file_node_text:
            # TODO: remove next hack
            if not context.settings[context.current_setting]['OUTPUT_DIRECTORY']:
                context.settings[context.current_setting]['OUTPUT_DIRECTORY'] = ['']
            if not context.settings[context.current_setting]['TARGET_NAME']:
                context.settings[context.current_setting]['TARGET_NAME'] = ['']

            output_path = context.settings[context.current_setting]['OUTPUT_DIRECTORY'][0]
            output_file = cleaning_output(context, output_file_node_text)
            output_file = output_file.replace('${OUTPUT_DIRECTORY}', output_path)
            output_path = os.path.dirname(output_file)
            name, _ = os.path.splitext(os.path.basename(output_file))
            name = name.replace(
                '${TARGET_NAME}',
                context.settings[context.current_setting]['TARGET_NAME'][0]
            )
            context.settings[context.current_setting]['TARGET_NAME'] = \
                [replace_vs_vars_with_cmake_vars(context, name)]

            output_path = check_for_relative_in_path(context, output_path)
            context.settings[context.current_setting]['OUTPUT_DIRECTORY'] = [output_path]

            message(
                context,
                'Output File : dir="{}" name="{}"'.format(
                    context.settings[context.current_setting]['OUTPUT_DIRECTORY'][0],
                    context.settings[context.current_setting]['TARGET_NAME']),
                '')

    @staticmethod
    def set_path_and_name_from_node(context, node_name, value, path_property, name_property):
        file_path_value = value.strip()
        if not file_path_value:
            return

        file_path_value = replace_vs_vars_with_cmake_vars(context, file_path_value)
        file_path_value = set_native_slash(file_path_value)
        path, name = get_dir_name_with_vars(context, file_path_value)
        result_name = replace_vs_vars_with_cmake_vars(context, name)
        result_path = cleaning_output(context, path)
        result_path = check_for_relative_in_path(context, result_path)

        message(context, '{} directory = {}'.format(node_name, result_path), '')
        message(context, '{} name = {}'.format(node_name, result_name), '')

        context.settings[context.current_setting][path_property] = [result_path]
        context.settings[context.current_setting][name_property] = [result_name]

    @staticmethod
    def write_target_property(cmake_file,
                              property_indent,
                              config_condition_expr,
                              property_value,
                              width,
                              **kwargs):
        width_diff = 0
        property_name = kwargs['property_name']

        config = ''
        if config_condition_expr is not None:
            config = '_' + config_condition_expr.replace('$<CONFIG:', '').replace('>', '')
        width_diff = len('$<CONFIG:>') - len('_')

        if property_value:
            for property_sheet_cmake in property_value:
                result_width = width - width_diff
                if result_width < 0:
                    result_width = 0
                cmake_file.write(
                    '{}    {}{:<{width}} "{}"\n'.format(
                        property_indent,
                        property_name,
                        config.upper(),
                        property_sheet_cmake,
                        width=result_width)
                )

    @staticmethod
    def write_target_outputs(context, cmake_file):
        """
        Add outputs for each artefacts CMake target

        :param context: related full context
        :type context: Context
        :param cmake_file: CMakeLists.txt IO wrapper
        :type cmake_file: _io.TextIOWrapper
        """

        if not context.settings:
            return

        if context.root_namespace:
            cmake_file.write('set(ROOT_NAMESPACE {})\n\n'.format(context.root_namespace))

        write_property_of_settings(
            cmake_file, context.settings,
            context.sln_configurations_map,
            begin_text='set_target_properties(${PROJECT_NAME} PROPERTIES',
            end_text=')',
            property_name='VS_GLOBAL_KEYWORD',
            write_setting_property_func=ProjectVariables.write_target_property
        )

        if is_settings_has_data(context.sln_configurations_map,
                                context.settings,
                                'TARGET_NAME'):
            write_comment(cmake_file, 'Target name')
            write_property_of_settings(
                cmake_file, context.settings,
                context.sln_configurations_map,
                begin_text='set_target_properties(${PROJECT_NAME} PROPERTIES',
                end_text=')',
                property_name='TARGET_NAME',
                write_setting_property_func=ProjectVariables.write_target_property
            )

        if is_settings_has_data(context.sln_configurations_map,
                                context.settings,
                                'OUTPUT_DIRECTORY'):
            write_comment(cmake_file, 'Output directory')
            write_property_of_settings(
                cmake_file, context.settings,
                context.sln_configurations_map,
                begin_text='set_target_properties(${PROJECT_NAME} PROPERTIES',
                end_text=')',
                property_name='OUTPUT_DIRECTORY',
                write_setting_property_func=ProjectVariables.write_target_property
            )

        write_property_of_settings(
            cmake_file, context.settings,
            context.sln_configurations_map,
            begin_text='set_target_properties(${PROJECT_NAME} PROPERTIES',
            end_text=')',
            property_name='ARCHIVE_OUTPUT_DIRECTORY',
            write_setting_property_func=ProjectVariables.write_target_property
        )

        write_property_of_settings(
            cmake_file, context.settings,
            context.sln_configurations_map,
            begin_text='set_target_properties(${PROJECT_NAME} PROPERTIES',
            end_text=')',
            property_name='ARCHIVE_OUTPUT_NAME',
            write_setting_property_func=ProjectVariables.write_target_property
        )

        write_property_of_settings(
            cmake_file, context.settings,
            context.sln_configurations_map,
            begin_text='set_target_properties(${PROJECT_NAME} PROPERTIES',
            end_text=')',
            property_name='PDB_OUTPUT_DIRECTORY',
            write_setting_property_func=ProjectVariables.write_target_property
        )
        # PDB_NAME doesn't support generator expressions yet (CMake 3.13)
        # write_property_of_settings(
        #     cmake_file, context.settings,
        #     context.sln_configurations_map,
        #     begin_text='set_target_properties(${PROJECT_NAME} PROPERTIES',
        #     end_text=')',
        #     property_name='PDB_NAME',
        #     write_setting_property_func=ProjectVariables.write_target_property
        # )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

"""
    Data Files
    ==========
     Manage the **VS Project** data and creation of **CMakeLists.txt** file
"""

import os
from lxml import etree

from cmake_converter.utils import message, get_actual_filename, set_unix_slash


def search_file_path(context, xml_file):
    xml_file = set_unix_slash(xml_file)

    found_xml_file = get_actual_filename(context, xml_file)

    if found_xml_file is None:
        message(
            context,
            '{0} file not exists. '.format(xml_file),
            'error'
        )
        return None

    if found_xml_file != xml_file:
        message(
            context,
            'file reference probably has wrong case {0}'.format(xml_file),
            'warn4'
        )

    return found_xml_file


def get_vcxproj_data(context, vs_project):
    """
    Return xml data from "vcxproj" file

    :param context: the context of converter
    :type context: Context
    :param vs_project: the vcxproj file
    :type vs_project: str
    :return: dict with VS Project data
    :rtype: dict
    """

    vcxproj = {}
    vs_project = search_file_path(context, vs_project)
    if vs_project is None:
        return None

    try:
        tree = etree.parse(vs_project)
        namespace = str(tree.getroot().nsmap)
        ns = {'ns': namespace.partition('\'')[-1].rpartition('\'')[0]}
        vcxproj['tree'] = tree
        vcxproj['ns'] = ns
        assert 'http://schemas.microsoft.com' in ns['ns']
    except AssertionError:  # pragma: no cover
        message(
            context,
            '{} file cannot be import, because this file does not seem to comply with'
            ' Microsoft xml data !'.format(vs_project),
            'error'
        )
        exit(1)
    except (OSError, IOError):  # pragma: no cover
        message(
            context,
            '{} file cannot be import. '
            'Please, verify you have rights to this directory or file exists !'.format(vs_project),
            'error'
        )
        exit(1)
    except etree.XMLSyntaxError:  # pragma: no cover
        message(
            context,
            'File {} is not a ".vcxproj" file or XML is broken !'.format(vs_project),
            'error'
        )
        exit(1)

    return vcxproj


def get_xml_data(context, xml_file):
    """
    Return xml data from "xml" file

    :param context: the context of converter
    :type context: Context
    :param xml_file: the xml file
    :type xml_file: str
    :return: dict with VS Project data
    :rtype: dict
    """

    xml = {}
    xml_file = search_file_path(context, xml_file)
    if xml_file is None:
        return None

    try:
        tree = etree.parse(xml_file)
        namespace = str(tree.getroot().nsmap)
        ns = {'ns': namespace.partition('\'')[-1].rpartition('\'')[0]}
        xml['tree'] = tree
        xml['ns'] = ns
        # assert 'http://schemas.microsoft.com' in ns['ns']
    except AssertionError:  # pragma: no cover
        message(
            context,
            '.xml file {} cannot be import, because this file does not seem to comply with'
            ' Microsoft xml data !'.format(xml_file),
            'error'
        )
        exit(1)
    except (OSError, IOError):  # pragma: no cover
        message(
            context,
            '{} file cannot be import. '
            'Please, verify you have rights to this directory or file exists !'.format(xml_file),
            'error'
        )
        exit(1)
    except etree.XMLSyntaxError:  # pragma: no cover
        message(
            context,
            'File {} is not a ".xml" file or XML is broken !'.format(xml_file),
            'error'
        )
        exit(1)

    return xml


def get_propertygroup(target_platform, attributes=''):
    """
    Return "property_groups" value for wanted platform and target

    :param target_platform: wanted target: debug | release
    :type target_platform: tuple[str,str]
    :param attributes: attributes to add to namespace
    :type attributes: str
    :return: "property_groups" value
    :rtype: str
    """

    prop = \
        '//ns:PropertyGroup[@Condition="\'$(Configuration)|$(Platform)\'==\'{0}\'"{1}]'.format(
            '|'.join(target_platform), attributes)

    return prop


def get_definitiongroup(target_platform):
    """
    Return ItemDefinitionGroup namespace depends on platform and target

    :param target_platform: wanted target: debug | release
    :type target_platform: tuple[str,str]
    :return: wanted ItemDefinitionGroup namespace
    :rtype: str
    """

    item = \
        '//ns:ItemDefinitionGroup[@Condition="\'$(Configuration)|' \
        '$(Platform)\'==\'{0}\'"]'.format('|'.join(target_platform))

    return item


def get_cmake_lists(context, cmake_path=None, open_type='w'):
    """
    Create CMakeLists.txt file in wanted "cmake_path"

    :param context: the context of converter
    :type context: Context
    :param cmake_path: path where CMakeLists.txt should be open
    :type cmake_path: str
    :param open_type: type that CMakeLists.txt should be opened
    :type open_type: str
    :return: cmake file wrapper opened
    :rtype: _io.TextIOWrapper
    """

    if not cmake_path:
        if open_type == 'w':
            message(context, 'CMakeLists will be written at current directory.', 'warn')
        cmake = os.path.join(os.getcwd(), 'CMakeLists.txt')
    else:
        if open_type == 'w':
            message(context, 'CMakeLists.txt will be written at : ' + str(cmake_path), '')
        if cmake_path[-1:] == '/' or cmake_path[-1:] == '\\':
            cmake = str(cmake_path) + 'CMakeLists.txt'
        else:
            cmake = str(cmake_path) + '/CMakeLists.txt'

    if not os.path.exists(cmake) and open_type == 'r':
        return None

    return open(cmake, open_type, newline='\n', encoding='utf-8')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

"""
    Main
    ====
     Manage script arguments and launch
"""

import argparse
import os

from cmake_converter.context import Context
from cmake_converter.visual_studio.solution import convert_solution
from cmake_converter.utils import message


def main():  # pragma: no cover
    """
    Define arguments and message to DataConverter()

    """

    usage = "cmake-converter -s <path/to/file.sln> " \
            "[ -h | -p | -d | -v | -w | -j | -a ]"
    parser = argparse.ArgumentParser(
        usage=usage,
        description='Converts Visual Studio projects in solution (*.sln) to CMakeLists.txt tree'
    )
    parser.add_argument(
        '-s', '--solution',
        help='[required] valid solution file. i.e.: ../../my.sln',
        required=True,
        dest='solution'
    )
    parser.add_argument(
        '-p', '--projects-filter',
        help='python regexp to filter that projects should be converted from the given solution',
        dest='projects_regexp'
    )
    parser.add_argument(
        '-d', '--dry-run',
        help='run converter without touching files.',
        dest='dry',
        action='store_true'
    )
    parser.add_argument(
        '-v', '--verbose-mode',
        help='run converter with more messages in log.',
        dest='verbose',
        action='store_true'
    )
    parser.add_argument(
        '-w', '--warning-level',
        help='run converter with given verbocity of warnings([1..4]default=3).',
        dest='warn',
    )
    parser.add_argument(
        '-j', '--jobs',
        help='run converter using given number of processes.',
        dest='jobs',
    )
    parser.add_argument(
        '-a', '--additional',
        help='[experimental] import cmake code from file.cmake to your final CMakeLists.txt',
        dest='additional'
    )

    parser.add_argument(
        '-pi', '--private-include-directories',
        help='use PRIVATE specifier for target_include_directories',
        dest='private_includes',
        default=False,
        action='store_true'
    )

    args = parser.parse_args()

    initial_context = Context()
    # Prepare context
    initial_context.additional_code = args.additional

    if args.projects_regexp:
        initial_context.projects_regexp = args.projects_regexp

    if args.dry:
        initial_context.dry = True
        message(initial_context, 'Converter runs in dry mode', 'done')

    if args.verbose:
        initial_context.verbose = True
        message(initial_context, 'Converter runs in verbose mode', 'done')

    if args.jobs:
        initial_context.jobs = int(args.jobs)
    message(initial_context, 'processes count = {}'. format(initial_context.jobs), 'done')

    if args.warn:
        initial_context.warn_level = int(args.warn)
    message(initial_context, 'warnings level = {}'. format(initial_context.warn_level), 'done')

    if args.private_includes:
        message(initial_context, 'include directories will be PRIVATE', 'done')
        initial_context.private_include_directories = True

    convert_solution(initial_context, os.path.abspath(args.solution))


if __name__ == "__main__":  # pragma: no cover
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

from cmake_converter.project_variables import ProjectVariables


class VCXProjectVariables(ProjectVariables):
    """
        Class who defines all the CMake variables to be used by the C/C++ project
    """

    @staticmethod
    def set_root_namespace(context, node):
        context.root_namespace = node.text

    @staticmethod
    def set_keyword(context, node):
        context.settings[context.current_setting]['VS_GLOBAL_KEYWORD'] = [node.text]

    @staticmethod
    def set_windows_target_version(context, node):
        context.target_windows_version = node.text

    def set_output_dir(self, context, node):
        self.set_output_dir_impl(context, node.text)

    def set_output_file(self, context, output_file_node):
        if output_file_node is not None:
            self.set_output_file_impl(context, output_file_node.text)

    def set_import_library(self, context, node):
        self.set_path_and_name_from_node(
            context,
            'Import library',
            node.text,
            'ARCHIVE_OUTPUT_DIRECTORY',
            'ARCHIVE_OUTPUT_NAME'
        )

    def set_program_database_file(self, context, node):
        self.set_path_and_name_from_node(
            context,
            'Program database',
            node.text,
            'PDB_OUTPUT_DIRECTORY',
            'PDB_NAME'
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

import os
from collections import OrderedDict

from cmake_converter.flags import Flags, default_value, ifort_cl_unix, ifort_cl_win, ifort_ln_win, \
    ifort_ln_unix
from cmake_converter.utils import normalize_path, set_unix_slash, message


class FortranFlags(Flags):
    """
        Class who check and create compilation flags for Fortran compiler
    """

    def __init__(self):
        self.flags = {}
        self.flags_handlers = OrderedDict([
            ('VFFortranCompilerTool_SuppressStartupBanner', self.__set_suppress_startup_banner),
            ('VFFortranCompilerTool_DebugInformationFormat', self.__set_debug_information_format),
            ('VFFortranCompilerTool_Optimization', self.__set_optimization),
            ('VFFortranCompilerTool_InterproceduralOptimizations',
             self.__set_interprocedural_optimizations),
            ('VFFortranCompilerTool_EnableEnhancedInstructionSet',
             self.__set_enable_enhanced_instruction_set),
            ('VFFortranCompilerTool_EnableRecursion', self.__set_enable_recursion),
            ('VFFortranCompilerTool_ReentrantCode', self.__set_reentrant_code),
            ('VFFortranCompilerTool_Preprocess', self.__set_preprocess_source_file),
            ('VFFortranCompilerTool_SourceFileFormat', self.__set_source_file_format),
            ('VFFortranCompilerTool_DebugParameter', self.__set_debug_parameter),
            ('VFFortranCompilerTool_DefaultIncAndUsePath', self.__set_default_inc_and_use_path),
            ('VFFortranCompilerTool_FixedFormLineLength', self.__set_fixed_form_line_length),
            ('VFFortranCompilerTool_OpenMP', self.__set_open_mp),
            ('VFFortranCompilerTool_DisableSpecificDiagnostics',
             self.__set_disable_specific_diagnostics),
            ('VFFortranCompilerTool_Diagnostics', self.__set_diagnostics),
            ('VFFortranCompilerTool_WarnDeclarations', self.__set_warn_declarations),
            ('VFFortranCompilerTool_WarnUnusedVariables', self.__set_warn_unused_variables),
            ('VFFortranCompilerTool_WarnIgnoreLOC', self.__set_warn_ignore_loc),
            ('VFFortranCompilerTool_WarnTruncateSource', self.__set_warn_truncate_source),
            ('VFFortranCompilerTool_WarnInterfaces', self.__set_warn_interfaces),
            ('VFFortranCompilerTool_WarnUnalignedData', self.__set_warn_unaligned_data),
            ('VFFortranCompilerTool_WarnUncalled', self.__set_warn_uncalled),
            ('VFFortranCompilerTool_SuppressUsageMessages', self.__set_suppress_usage_messages),
            ('VFFortranCompilerTool_RealKIND', self.__set_real_kind),
            ('VFFortranCompilerTool_LocalVariableStorage', self.__set_local_variable_storage),
            ('VFFortranCompilerTool_InitLocalVarToNAN', self.__set_init_local_var_to_nan),
            ('VFFortranCompilerTool_FloatingPointExceptionHandling',
             self.__set_floating_point_exception_handling),
            ('VFFortranCompilerTool_ExtendSinglePrecisionConstants',
             self.__set_extend_single_precision_constants),
            ('VFFortranCompilerTool_FloatingPointModel', self.__set_floating_point_model),
            ('VFFortranCompilerTool_FloatingPointSpeculation',
             self.__set_floating_point_speculation),
            ('VFFortranCompilerTool_FloatingPointStackCheck',
             self.__set_floating_point_stack_check),
            ('VFFortranCompilerTool_ExternalNameInterpretation',
             self.__set_external_name_interpretation),
            ('VFFortranCompilerTool_CallingConvention', self.__set_calling_convention),
            ('VFFortranCompilerTool_StringLengthArgPassing', self.__set_string_length_arg_passing),
            ('VFFortranCompilerTool_ExternalNameUnderscore', self.__set_external_name_underscore),
            ('VFFortranCompilerTool_Traceback', self.__set_traceback),
            ('VFFortranCompilerTool_RuntimeChecks', self.__set_runtime_checks),
            ('VFFortranCompilerTool_NullPointerCheck', self.__set_null_pointer_check),
            ('VFFortranCompilerTool_BoundsCheck', self.__set_bounds_check),
            ('VFFortranCompilerTool_UninitializedVariablesCheck',
             self.__set_uninitialized_variables_check),
            ('VFFortranCompilerTool_DescriptorDataTypeCheck',
             self.__set_descriptor_data_type_check),
            ('VFFortranCompilerTool_DescriptorDataSizeCheck',
             self.__set_descriptor_data_size_check),
            ('VFFortranCompilerTool_ArgTempCreatedCheck', self.__set_arg_temp_created_check),
            ('VFFortranCompilerTool_StackFrameCheck', self.__set_stack_frame_check),
            ('VFFortranCompilerTool_RuntimeLibrary', self.__set_runtime_library),
            ('VFFortranCompilerTool_DisableDefaultLibSearch',
             self.__set_disable_default_lib_search),
            ('VFFortranCompilerTool_AdditionalOptions', self.__set_additional_options),
            ('VFLinkerTool_GenerateDebugInformation', self.__generate_debug_information),
            ('VFLinkerTool_LinkIncremental', self.__set_link_incremental),
            ('VFLinkerTool_SuppressStartupBanner', self.__set_link_suppress_startup_banner),
            ('VFLinkerTool_SubSystem', self.__set_sub_system),
            ('VFLinkerTool_LinkDLL', self.__set_link_dll),
            ('VFLinkerTool_AdditionalOptions', self.__set_additional_link_options),
        ])

    def __set_default_flags(self, context):
        for flag_name in self.flags_handlers:
            self.flags[flag_name] = {}
            self.set_flag(context, flag_name, '', None)

    def set_flag(self, context, flag_name, flag_value, node):
        """
        Set flag helper

        :param context: converter Context
        :type context: Context
        :param flag_name:
        :param flag_value:
        :param node:
        :return:
        """
        del node
        if None in context.current_setting:
            return

        flag_values = self.flags_handlers[flag_name](context, flag_name, flag_value)

        if flag_values is None:
            return

        values = None
        if default_value in flag_values:
            values = flag_values[default_value]

        if flag_value in flag_values:
            values = flag_values[flag_value]

        flags_message = {}
        if values is not None:
            self.flags[flag_name] = {}  # reset default values
            for key in values:
                value = values[key]
                self.flags[flag_name][key] = [value]
                flags_message[key] = value

        if flags_message:
            message(
                context,
                '{0} is {1} '.format(flag_name, flags_message),
                ''
            )

    def prepare_context_for_flags(self, context):
        context.flags.flags[context.current_setting] = {}
        self.__set_default_flags(context)

    def apply_flags_to_context(self, context):
        context_flags_data_keys = [
            ifort_cl_win,
            ifort_cl_unix,
            ifort_ln_win,
            ifort_ln_unix,
            'assume_args',
            'warn_args',
            'check_args'
        ]

        for flag_name in self.flags_handlers:
            for context_flags_data_key in context_flags_data_keys:
                if flag_name not in self.flags:
                    continue
                if context_flags_data_key in self.flags[flag_name]:
                    for value in self.flags[flag_name][context_flags_data_key]:
                        context.settings[context.current_setting][context_flags_data_key].append(
                            value
                        )

        self.__set_spec_options(context, 'assume_args', 'assume')
        self.__set_spec_options(context, 'check_args', 'check')
        self.__set_spec_options(context, 'warn_args', 'warn')

    @staticmethod
    def __set_spec_options(context, context_key, compiler_key):
        if context_key in context.settings[context.current_setting]:
            args = context.settings[context.current_setting][context_key]
            if args:
                context.settings[context.current_setting][ifort_cl_win].append(
                    '-{}:{}'.format(compiler_key, ','.join(args))
                )
                context.settings[context.current_setting][ifort_cl_unix].append(
                    '-{} {}'.format(compiler_key, ','.join(args))
                )

    @staticmethod
    def __set_floating_point_stack_check(context, flag_name, flag_value):
        del context, flag_name, flag_value
        flag_values = {
            'true': {ifort_cl_win: '-Qfp-stack-check',
                     ifort_cl_unix: '-fp-stack-check'},
            'false': {},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_external_name_underscore(context, flag_name, flag_value):
        if context.file_contexts is None and flag_value == '':
            return {}

        del flag_name
        flag_values = {
            'true': {'assume_args': 'underscore'},
            'false': {'assume_args': 'nounderscore'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_external_name_interpretation(context, flag_name, flag_value):
        del context, flag_name, flag_value
        flag_values = {
            'extNameUpperCase': {ifort_cl_win: '-names:uppercase',
                                 ifort_cl_unix: '-names uppercase'},
            'extNameLowerCase': {ifort_cl_win: '-names:lowercase',
                                 ifort_cl_unix: '-names lowercase'},
            'extNameAsIs': {ifort_cl_win: '-names:as_is',
                            ifort_cl_unix: '-names as_is'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_calling_convention(context, flag_name, flag_value):
        del context, flag_name, flag_value
        flag_values = {
            'callConventionCRef': {ifort_cl_win: '-iface:cref'},
            'callConventionStdRef': {ifort_cl_win: '-iface:stdref'},
            'callConventionStdCall': {ifort_cl_win: '-iface:stdcall'},
            'callConventionCVF': {ifort_cl_win: '-iface:cvf'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_diagnostics(context, flag_name, flag_value):
        del context, flag_name, flag_value
        flag_values = {
            'diagnosticsShowAll': {ifort_cl_win: '-warn:all',
                                   ifort_cl_unix: '-warn all'},
            'diagnosticsDisableAll': {ifort_cl_win: '-warn:none',
                                      ifort_cl_unix: '-warn none'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_warn_declarations(context, flag_name, flag_value):
        del context, flag_name, flag_value
        flag_values = {
            'true': {'warn_args': 'declaration'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_warn_unused_variables(context, flag_name, flag_value):
        del context, flag_name, flag_value
        flag_values = {
            'true': {'warn_args': 'unused'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_warn_ignore_loc(context, flag_name, flag_value):
        del context, flag_name, flag_value
        flag_values = {
            'true': {'warn_args': 'ignore_loc'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_warn_truncate_source(context, flag_name, flag_value):
        del context, flag_name, flag_value
        flag_values = {
            'true': {'warn_args': 'truncated_source'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_warn_interfaces(context, flag_name, flag_value):
        del context, flag_name, flag_value
        flag_values = {
            'true': {'warn_args': 'interfaces'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_warn_unaligned_data(context, flag_name, flag_value):
        del context, flag_name, flag_value
        flag_values = {
            'false': {'warn_args': 'noalignments'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_warn_uncalled(context, flag_name, flag_value):
        del context, flag_name, flag_value
        flag_values = {
            'true': {'warn_args': 'uncalled'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_suppress_usage_messages(context, flag_name, flag_value):
        del context, flag_name, flag_value
        flag_values = {
            'true': {'warn_args': 'nousage'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_null_pointer_check(context, flag_name, flag_value):
        """
        Set check:pointer
        """
        del context, flag_name, flag_value
        flag_values = {
            'true': {'check_args': 'pointer'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_bounds_check(context, flag_name, flag_value):
        """
        Set check:bounds
        """
        del context, flag_name, flag_value
        flag_values = {
            'true': {'check_args': 'bounds'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_uninitialized_variables_check(context, flag_name, flag_value):
        """
        Set check:uninit
        """
        del context, flag_name, flag_value
        flag_values = {
            'true': {'check_args': 'uninit'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_descriptor_data_type_check(context, flag_name, flag_value):
        """
        Set check:format
        """
        del context, flag_name, flag_value
        flag_values = {
            'true': {'check_args': 'format'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_descriptor_data_size_check(context, flag_name, flag_value):
        """
        Set check:output_conversion
        """
        del context, flag_name, flag_value
        flag_values = {
            'true': {'check_args': 'output_conversion'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_arg_temp_created_check(context, flag_name, flag_value):
        """
        Set check:arg_temp_created
        """
        del context, flag_name, flag_value
        flag_values = {
            'true': {'check_args': 'arg_temp_created'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_stack_frame_check(context, flag_name, flag_value):
        """
        Set check:stack
        """
        del context, flag_name, flag_value
        flag_values = {
            'true': {'check_args': 'stack'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_debug_parameter(context, flag_name, flag_value):
        """
        Set check:debug-parameters
        """
        del context, flag_name, flag_value
        flag_values = {
            'debugParameterAll': {ifort_cl_win: '-debug-parameters:all',
                                  ifort_cl_unix: '-debug-parameters all'},
            'debugParameterUsed': {ifort_cl_win: '-debug-parameters:used',
                                   ifort_cl_unix: '-debug-parameters used'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_fixed_form_line_length(context, flag_name, flag_value):
        """
        Set fixed form line length

        """
        del context, flag_name, flag_value
        flag_values = {
            'fixedLength80': {ifort_cl_win: '-extend-source:80',
                              ifort_cl_unix: '-extend-source 80'},
            'fixedLength132': {ifort_cl_win: '-extend-source:132',
                               ifort_cl_unix: '-extend-source 132'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_default_inc_and_use_path(context, flag_name, flag_value):
        """
        Set default include and path

        """
        del context, flag_name, flag_value
        flag_values = {
            'defaultIncludeCurrent': {'assume_args': 'nosource_include'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_open_mp(context, flag_name, flag_value):
        """
        Set open MP flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'OpenMPParallelCode': {ifort_cl_win: '-Qopenmp',
                                   ifort_cl_unix: '-qopenmp'},
            'OpenMPSequentialCode': {ifort_cl_win: '-Qopenmp-stubs',
                                     ifort_cl_unix: '-qopenmp-stubs'},
            default_value: {}
        }
        return flag_values

    def __set_disable_specific_diagnostics(self, context, flag_name, flag_value):
        """
        Set disable specific diagnostic flag

        """
        del context
        # value must be a list with ',' separator
        opt = flag_value
        if opt:
            self.flags[flag_name][ifort_cl_win] = ['-Qdiag-disable:{0}'.format(opt)]
            self.flags[flag_name][ifort_cl_unix] = ['-diag-disable={0}'.format(opt)]

    @staticmethod
    def __set_string_length_arg_passing(context, flag_name, flag_value):
        """
        Set string lengh arg parsing

        """
        del context, flag_name, flag_value
        flag_values = {
            'strLenArgsMixed': {ifort_cl_win: '-iface:mixed_str_len_arg',
                                ifort_cl_unix: '-mixed-str-len-arg'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_runtime_library(context, flag_name, flag_value):
        """
        Set runtime library flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'rtMultiThreadedDLL': {ifort_cl_win: '-libs:dll;-threads',
                                   ifort_cl_unix: '-threads'},
            'rtQuickWin': {ifort_cl_win: '-libs:qwin'},
            'rtStandardGraphics': {ifort_cl_win: '-libs:qwins'},
            'rtMultiThreadedDebug': {ifort_cl_win: '-libs:static;-threads;-dbglibs',
                                     ifort_cl_unix: '-threads'},
            'rtMultiThreadedDebugDLL': {ifort_cl_win: '-libs:dll;-threads;-dbglibs',
                                        ifort_cl_unix: '-threads'},
            'rtQuickWinDebug': {ifort_cl_win: '-libs:qwin;-dbglibs'},
            'rtStandardGraphicsDebug': {ifort_cl_win: '-libs:qwins;-dbglibs'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_disable_default_lib_search(context, flag_name, flag_value):
        """
        Set disable default lib search

        """
        del context, flag_name, flag_value
        flag_values = {
            'true': {ifort_cl_win: '-libdir:noauto'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_runtime_checks(context, flag_name, flag_value):
        """
        Set runtime checks flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'rtChecksAll': {ifort_cl_win: '-check:all',
                            ifort_cl_unix: '-check all'},
            'rtChecksNone': {ifort_cl_win: '-nocheck',
                             ifort_cl_unix: '-nocheck'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_traceback(context, flag_name, flag_value):
        """
        Set traceback flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'true': {ifort_cl_win: '-traceback',
                     ifort_cl_unix: '-traceback'},
            'false': {ifort_cl_win: '-notraceback',
                      ifort_cl_unix: '-notraceback'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_extend_single_precision_constants(context, flag_name, flag_value):
        """
        Set extend single precision constants flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'true': {ifort_cl_win: '-fpconstant',
                     ifort_cl_unix: '-fpconstant'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_floating_point_model(context, flag_name, flag_value):
        """
        Set extend single precision constants flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'fast2': {ifort_cl_win: '-fp:fast=2',
                      ifort_cl_unix: '-fp-model fast=2'},
            'strict': {ifort_cl_win: '-fp:strict',
                       ifort_cl_unix: '-fp-model strict'},
            'source': {ifort_cl_win: '-fp:source',
                       ifort_cl_unix: '-fp-model source'},
            'precise': {ifort_cl_win: '-fp:precise',
                        ifort_cl_unix: '-fp-model precise'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_floating_point_speculation(context, flag_name, flag_value):
        """
        Set extend single precision constants flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'fpSpeculationSafe': {ifort_cl_win: '-Qfp-speculation:safe',
                                  ifort_cl_unix: '-fp-speculation=safe'},
            'fpSpeculationStrict': {ifort_cl_win: '-Qfp-speculation:strict',
                                    ifort_cl_unix: '-fp-speculation=strict'},
            'fpSpeculationOff': {ifort_cl_win: '-Qfp-speculation:off',
                                 ifort_cl_unix: '-fp-speculation=off'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_floating_point_exception_handling(context, flag_name, flag_value):
        """
        Set floating exception handling

        """
        del context, flag_name, flag_value
        flag_values = {
            'fpe0': {ifort_cl_win: '-fpe0',
                     ifort_cl_unix: '-fpe0'},
            'fpe1': {ifort_cl_win: '-fpe1',
                     ifort_cl_unix: '-fpe1'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_init_local_var_to_nan(context, flag_name, flag_value):
        """
        Set init local var to NaN flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'true': {ifort_cl_win: '-Qtrapuv',
                     ifort_cl_unix: '-ftrapuv'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_preprocess_source_file(context, flag_name, flag_value):
        """
        Set preprocess source file flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'preprocessYes': {ifort_cl_win: '-fpp',
                              ifort_cl_unix: '-fpp'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_optimization(context, flag_name, flag_value):
        """
        Set optimization flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'optimizeMinSpace': {ifort_cl_win: '-O1',
                                 ifort_cl_unix: '-O1'},
            'optimizeFull': {ifort_cl_win: '-O3',
                             ifort_cl_unix: '-O3'},
            'optimizeDisabled': {ifort_cl_win: '-Od',
                                 ifort_cl_unix: '-O0'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_interprocedural_optimizations(context, flag_name, flag_value):
        """
        Set InterproceduralOptimizations

        """
        del flag_name, flag_value

        target_type = context.settings[context.current_setting]['target_type']
        fat_lto_option = ''
        if target_type == 'StaticLibrary':
            fat_lto_option = ';-ffat-lto-objects'
            message(
                context,
                'For unix added option -ffat-lto-objects to fix linking with -ipo.',
                ''
            )

        flag_values = {
            'ipoMultiFile': {ifort_cl_win: '-Qipo',
                             ifort_cl_unix: '-ipo{}'.format(fat_lto_option)},
            'ipoSingleFile': {ifort_cl_win: '-Qip',
                              ifort_cl_unix: '-ip'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_enable_enhanced_instruction_set(context, flag_name, flag_value):
        """
        Set EnableEnhancedInstructionSet

        """
        del context, flag_name, flag_value
        flag_values = {
            'codeArchSSE2': {ifort_cl_win: '-arch:SSE2',
                             ifort_cl_unix: '-arch SSE2'},
            'codeArchSSE3': {ifort_cl_win: '-arch:SSE3',
                             ifort_cl_unix: '-arch SSE3'},
            'codeArchAVX': {ifort_cl_win: '-arch:AVX',
                            ifort_cl_unix: '-arch AVX'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_enable_recursion(context, flag_name, flag_value):
        del context, flag_name, flag_value
        flag_values = {
            'true': {ifort_cl_win: '-recursive',
                     ifort_cl_unix: '-recursive'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_reentrant_code(context, flag_name, flag_value):
        del context, flag_name, flag_value
        flag_values = {
            'reentrancyNone': {ifort_cl_win: '-reentrancy:none',
                               ifort_cl_unix: '-reentrancy none'},
            'reentrancyAsync': {ifort_cl_win: '-reentrancy:async',
                                ifort_cl_unix: '-reentrancy async'},
            'reentrancyThreaded': {ifort_cl_win: '-reentrancy:threaded',
                                   ifort_cl_unix: '-reentrancy threaded'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_debug_information_format(context, flag_name, flag_value):
        """
        Set debug information format flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'debugEnabled': {ifort_cl_win: '-debug:full',
                             ifort_cl_unix: '-debug full'},
            'debugLineInfoOnly': {ifort_cl_win: '-debug:minimal',
                                  ifort_cl_unix: '-debug minimal'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_suppress_startup_banner(context, flag_name, flag_value):
        """
        Set suppress banner flag for compiler

        """
        del context, flag_name, flag_value
        flag_values = {
            'true': {ifort_cl_win: '-nologo'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_source_file_format(context, flag_name, flag_value):
        """
        Set source file format flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'fileFormatFree': {ifort_cl_win: '-free',
                               ifort_cl_unix: '-free'},
            'fileFormatFixed': {ifort_cl_win: '-fixed',
                                ifort_cl_unix: '-fixed'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_local_variable_storage(context, flag_name, flag_value):
        """
        Set local variable storage flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'localStorageAutomatic': {ifort_cl_win: '-Qauto',
                                      ifort_cl_unix: '-auto'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_real_kind(context, flag_name, flag_value):
        """
        Set real kind flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'realKIND8': {ifort_cl_win: '-real-size:64',
                          ifort_cl_unix: '-real-size 64'},
            'realKIND16': {ifort_cl_win: '-real-size:128',
                           ifort_cl_unix: '-real-size 128'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __get_no_prefix(unix_option):
        no = ''
        if unix_option[-1:] == '-':
            no = '-no'
        return no

    def __set_additional_options(self, context, flag_name, flag_value):
        """
        Set Additional options

        """
        # for setting in context.settings:
        add_opts = flag_value
        if add_opts:
            add_opts = set_unix_slash(add_opts).split()
            ready_add_opts = []
            for add_opt in add_opts:
                add_opt = add_opt.strip()
                if '/Qprof-dir' in add_opt:
                    name_value = add_opt.split(':')
                    add_opt = name_value[0] + ':' + normalize_path(
                        context,
                        os.path.dirname(context.vcxproj_path), name_value[1]
                    )

                add_opt = '-' + add_opt[1:]
                ready_add_opts.append(add_opt)
                unix_option = add_opt.replace(':', ' ')
                if 'gen-interfaces' in unix_option:
                    pass
                elif 'Qprec-div' in unix_option:
                    unix_option = FortranFlags.__get_no_prefix(unix_option) + '-prec-div'
                elif unix_option == '-static':
                    pass
                elif 'Qprof-dir' in unix_option:
                    unix_option = unix_option.replace('Qprof-dir', 'prof-dir')
                elif 'Qprof-use' in unix_option:
                    unix_option = unix_option.replace('Qprof-use', 'prof-use')
                elif 'Qprec-sqrt' in unix_option:
                    unix_option = FortranFlags.__get_no_prefix(unix_option) + '-prec-sqrt'
                elif 'Qopenmp-lib' in unix_option:
                    unix_option = unix_option.replace('Qopenmp-lib', 'qopenmp-lib')
                    unix_option = unix_option.replace('lib ', 'lib=')
                else:
                    message(context, 'Unix ifort option "{0}" may be incorrect. '
                            'Check it and set it with visual studio UI if possible.'
                            .format(unix_option), 'warn')
                if ifort_cl_win not in self.flags[flag_name]:
                    self.flags[flag_name][ifort_cl_win] = []
                self.flags[flag_name][ifort_cl_win].append(add_opt)
                if ifort_cl_unix not in self.flags[flag_name]:
                    self.flags[flag_name][ifort_cl_unix] = []
                self.flags[flag_name][ifort_cl_unix].append(unix_option)
            message(context,
                    'Additional Options : {0}'.format(str(ready_add_opts)), '')

    @staticmethod
    def __generate_debug_information(context, flag_name, flag_value):
        del context, flag_name, flag_value
        flag_values = {
            'true': {ifort_ln_win: '/DEBUG'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_link_incremental(context, flag_name, flag_value):
        """
        Set LinkIncremental flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'linkIncrementalYes': {ifort_ln_win: '/INCREMENTAL'},
            'linkIncrementalNo': {ifort_ln_win: '/INCREMENTAL:NO'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_link_suppress_startup_banner(context, flag_name, flag_value):
        """
        Set suppress banner flag for linker

        """
        del context, flag_name, flag_value
        flag_values = {
            'true': {ifort_ln_win: '/NOLOGO'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_sub_system(context, flag_name, flag_value):
        """
        Set SubSystem flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'subSystemWindows': {ifort_ln_win: '/SUBSYSTEM:WINDOWS'},
            'subSystemConsole': {ifort_ln_win: '/SUBSYSTEM:CONSOLE'},
            'subSystemConsoleXP': {ifort_ln_win: '/SUBSYSTEM:CONSOLE,"5.x"'},
            'subSystemWindowsXP': {ifort_ln_win: '/SUBSYSTEM:WINDOWS,"5.x"'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_link_dll(context, flag_name, flag_value):
        """
        Set LinkDLL flag

        """
        del context, flag_name, flag_value
        flag_values = {
            'true': {ifort_ln_win: '/DLL'},
            default_value: {}
        }
        return flag_values

    @staticmethod
    def __set_additional_link_options(context, flag_name, flag_value):
        """
        Set LinkDLL flag

        """
        del context, flag_name

        flag_values = {}
        if flag_value:
            flag_values.update(
                {
                    flag_value: {ifort_ln_win: flag_value},
                    default_value: {}
                }
            )
        return flag_values



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

import os

from cmake_converter.utils import message


class ContextInitializer:
    def __init__(self, context, vs_project, cmake_lists_destination_path):
        self.init_files(context, vs_project, cmake_lists_destination_path)
        message(
            context,
            'Initialization data for conversion of project {0}'.format(context.vcxproj_path),
            ''
        )
        for sln_config in context.sln_configurations_map:
            context.configurations_to_parse.add(context.sln_configurations_map[sln_config])

    def init_context(self, context, vs_project):
        """
        Initialize context

        """
        pass

    def init_files(self, context, vs_project, cmake_lists):
        """
        Initialize opening of CMakeLists.txt and VS Project files

        :param context: converter context
        :type context: Context
        :param vs_project: Visual Studio project file path
        :type vs_project: str
        :param cmake_lists: CMakeLists.txt file path
        :type cmake_lists: str
        """

        if vs_project:
            temp_path = os.path.splitext(vs_project)
            project_name = os.path.basename(temp_path[0])
            context.project_name = project_name
            context.vcxproj_path = vs_project
            self.init_context(context, vs_project)
            self.set_cmake_lists_path(context, cmake_lists)

    @staticmethod
    def set_cmake_lists_path(context, cmake_lists):
        """
        Set CMakeLists.txt path in context, for given project

        :param context: converter context
        :type context: Context
        :param cmake_lists: path of CMakeLists related to project name
        :type cmake_lists: str
        """

        context_cmake = None

        if cmake_lists:
            if os.path.exists(cmake_lists):
                context_cmake = cmake_lists

        if context_cmake is None:
            message(
                context,
                'Path "{}" for CMakeLists.txt is wrong. '
                'It will be created in working directory.'.format(cmake_lists),
                'warn'
            )
            context_cmake = 'CMakeLists.txt'

        if context:
            context.cmake = context_cmake

        return context_cmake



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

import re
import os
import shutil
from collections import OrderedDict
from multiprocessing import Pool

from cmake_converter.data_converter import DataConverter
from cmake_converter.context_initializer import ContextInitializer
from cmake_converter.data_files import get_cmake_lists
from cmake_converter.utils import set_unix_slash, message, write_comment, write_use_package_stub,\
    write_arch_types


def run_conversion(subdirectory_projects_data):
    results = []
    for project_data in subdirectory_projects_data:
        project_context = project_data['project_context']
        name = project_context.project_number
        message(project_context, '------ Starting {} -------'.format(name), '')
        converted = convert_project(
            project_context,
            project_data['project_abs'],
            project_data['subdirectory'],
        )
        message(project_context, '------ Exiting  {} -------'.format(name), '')

        if not converted:
            continue

        project_context = project_data['project_context']
        # Can't return context as a result due PicklingError
        results.append(
            {
                'cmake': project_context.cmake,
                'project_name': project_context.project_name,
                'solution_languages': project_context.solution_languages,
                'target_windows_ver': project_context.target_windows_version,
                'warnings_count': project_context.warnings_count
            }
        )
    return results


def parse_solution(initial_context, sln_text):
    """
    Parse given solution

    :param sln_text: full solution text
    :type sln_text: str
    :return: data from solution
    :rtype: dict
    """

    solution_data = {}
    projects_data = OrderedDict()
    solution_folders = {}

    __check_solution_version(initial_context, sln_text)

    __parse_projects_data(sln_text, solution_folders, projects_data)

    __parse_configurations_of_solution(initial_context, sln_text, solution_data)

    __parse_project_configuration_platforms(sln_text, projects_data)

    solution_folders_map = {}

    __parse_nested_projects_in_solution_folders(sln_text, solution_folders_map)

    set_solution_dirs_to_projects(projects_data, solution_folders_map, solution_folders)

    # replace GUIDs with Project names in dependencies
    for project_guid in projects_data:
        project_data = projects_data[project_guid]
        if 'sln_deps' in project_data:
            target_deps = []
            dependencies_list = project_data['sln_deps']
            for dep_guid in dependencies_list:
                dep = projects_data[dep_guid]
                target_deps.append(dep['name'])
            project_data['sln_deps'] = target_deps
    solution_data['projects_data'] = projects_data

    return solution_data


def __check_solution_version(initial_context, sln_text):
    version_pattern = re.compile(
        r'Microsoft Visual Studio Solution File, Format Version (.*)'
    )
    version_match = version_pattern.findall(sln_text)
    if not version_match or float(version_match[0]) < 9:
        message(initial_context, 'Solution files with versions below 9.00 are not supported.'
                                 ' Version {} found. Upgrade you solution and try again, please'
                .format(version_match[0]), 'error')
        exit(1)

    message(initial_context, 'Version of solution is {}'.format(version_match[0]), '')


def __parse_projects_data(sln_text, solution_folders, projects_data):
    p = re.compile(
        r'(Project.*\s=\s\"(.*)\",\s\"(.*)\",.*({.*\})(?:.|\n)*?EndProject(?!Section))'
    )

    for project_data_match in p.findall(sln_text):
        path = project_data_match[2]
        guid = project_data_match[3]

        _, ext = os.path.splitext(os.path.basename(path))

        if 'proj' not in ext:
            solution_folders[guid] = path
            continue

        project = dict()
        project['name'] = project_data_match[1]
        project['path'] = path
        if 'ProjectDependencies' in project_data_match[0]:
            project['sln_deps'] = []
            dependencies_section = re.compile(
                r'ProjectSection\(ProjectDependencies\) = postProject(?:.|\n)*?EndProjectSection'
            )
            dep_data = dependencies_section.findall(project_data_match[0])
            dependencies_guids = re.compile(r'(({.*\}) = ({.*\}))')
            guids_deps_matches = dependencies_guids.findall(dep_data[0])
            for guids_deps_match in guids_deps_matches:
                project['sln_deps'].append(guids_deps_match[2])
        projects_data[guid] = project


def __parse_configurations_of_solution(initial_context, sln_text, solution_data):
    solution_configurations_re = re.compile(
        r'GlobalSection\(SolutionConfigurationPlatforms\) = preSolution((?:.|\n)*?)EndGlobalSection'
    )

    solution_configurations_matches = solution_configurations_re.findall(sln_text)
    solution_data['sln_configurations'] = []
    sln_configuration_re = re.compile(r'([\w|]+) = ([\w|]+)')
    for solution_configuration_match in solution_configurations_matches:
        configurations = sln_configuration_re.findall(solution_configuration_match)
        for configuration in configurations:
            solution_data['sln_configurations'].append(configuration[0])
            arch = configuration[0].split('|')[1]
            if arch == 'x86':
                message(
                    initial_context,
                    'Solution architecture is x86 and may be mapped onto Win32 at projects.'
                    'To avoid problems rename x86 -> Win32.',
                    'warn')
            initial_context.supported_architectures.add(arch)


def __parse_project_configuration_platforms(sln_text, projects_data):
    projects_configurations_re = re.compile(
        r'GlobalSection\(ProjectConfigurationPlatforms\) = postSolution((?:.|\n)*?)EndGlobalSection'
    )
    projects_configurations_matches = projects_configurations_re.findall(sln_text)
    projects_configuration_re = re.compile(r'({.+\})\.([\w|]+)\.ActiveCfg = ([\w|]+)')
    for projects_configuration_match in projects_configurations_matches:
        configurations = projects_configuration_re.findall(projects_configuration_match)
        for configuration in configurations:
            p = projects_data[configuration[0]]
            if 'sln_configs_2_project_configs' not in p:
                p['sln_configs_2_project_configs'] = OrderedDict({(None, None): (None, None)})
            p['sln_configs_2_project_configs'][tuple(configuration[1].split('|'))] = \
                tuple(configuration[2].split('|'))


def __parse_nested_projects_in_solution_folders(sln_text, solution_folders_map):
    nested_projects_re = re.compile(
        r'GlobalSection\(NestedProjects\) = preSolution((?:.|\n)*?)EndGlobalSection'
    )
    nested_projects_matches = nested_projects_re.findall(sln_text)
    solution_dir_link_re = re.compile(r'(\{.+\}) = (\{.+\})')
    for nested_projects_match in nested_projects_matches:
        solution_dir_links = solution_dir_link_re.findall(nested_projects_match)
        for solution_dir_link in solution_dir_links:
            solution_folders_map[solution_dir_link[0]] = solution_dir_link[1]


def set_solution_dirs_to_projects(projects_data, solution_folders_map, solution_folders):
    for project_guid in projects_data:
        project_solution_dir = ''

        if project_guid in solution_folders_map:
            guid = project_guid
            while guid in solution_folders_map:
                if project_solution_dir:
                    project_solution_dir = '/' + project_solution_dir
                project_solution_dir = solution_folders[solution_folders_map[guid]]\
                    + project_solution_dir
                guid = solution_folders_map[guid]

        projects_data[project_guid]['project_solution_dir'] = project_solution_dir


def convert_project(context, xml_project_path, cmake_lists_destination_path):
    """
    Convert a ``vcxproj`` to a ``CMakeLists.txt``

    :param context: input data of user
    :type context: Context
    :param xml_project_path: input xml_proj
    :type xml_project_path: str
    :param cmake_lists_destination_path: Destination folder of CMakeLists.txt
    :type cmake_lists_destination_path: str
    """

    # Initialize Context of DataConverter
    if not context.init(xml_project_path, cmake_lists_destination_path):
        message(context, 'Unknown project type at {0}'.format(xml_project_path), 'error')
        return False

    data_converter = DataConverter()
    data_converter.convert(context)
    return True


def set_dependencies_for_project(context, project_data):
    if 'sln_deps' not in project_data:
        return

    context.sln_deps = project_data['sln_deps']


def clean_cmake_lists_file(context, subdirectory, cmake_lists_set):
    cmake_path_to_clean = \
        ContextInitializer.set_cmake_lists_path(context, subdirectory) + '/CMakeLists.txt'
    if context.dry:
        return

    if cmake_path_to_clean in cmake_lists_set:
        return
    cmake_lists_set.add(cmake_path_to_clean)

    if os.path.exists(cmake_path_to_clean):
        os.remove(cmake_path_to_clean)
        message(context, 'removed {}'.format(cmake_path_to_clean), '')
    else:
        message(context, 'not found {}'.format(cmake_path_to_clean), 'warn')


def clean_cmake_lists_of_solution(context, projects_data):
    if not os.path.exists(os.path.join(context.solution_path, 'CMake')):
        return  # first run

    message(context, 'Cleaning CMake Scripts', '')
    cmake_lists_set = set()
    for guid in projects_data:
        project_path = projects_data[guid]['path']
        project_path = '/'.join(project_path.split('\\'))
        project_abs = os.path.join(context.solution_path, project_path)
        subdirectory = os.path.dirname(project_abs)
        clean_cmake_lists_file(context, subdirectory, cmake_lists_set)

    clean_cmake_lists_file(context, context.solution_path, cmake_lists_set)
    print('\n')


def copy_cmake_utils(cmake_lists_path):
    utils_path = os.path.join(cmake_lists_path, 'CMake')
    if not os.path.exists(utils_path):
        os.makedirs(utils_path)
    src_dir = os.path.dirname(os.path.abspath(__file__))
    shutil.copyfile(os.path.join(src_dir, '../utils.cmake'), utils_path + '/Utils.cmake')
    shutil.copyfile(os.path.join(src_dir, '../Default.cmake'), utils_path + '/Default.cmake')
    shutil.copyfile(os.path.join(src_dir, '../DefaultCXX.cmake'), utils_path + '/DefaultCXX.cmake')
    shutil.copyfile(os.path.join(src_dir, '../DefaultFortran.cmake'),
                    utils_path + '/DefaultFortran.cmake')


def convert_solution(initial_context, sln_path):

    with open(sln_path, encoding='utf8') as sln:
        solution_data = parse_solution(initial_context, sln.read())

    initial_context.solution_path = os.path.dirname(sln_path)
    subdirectories_set = set()
    subdirectories_to_project_name = {}
    projects_data = solution_data['projects_data']

    clean_cmake_lists_of_solution(initial_context, projects_data)

    input_data_for_converter = __get_input_data_for_converter(
        initial_context,
        projects_data
    )

    results = __do_conversion(initial_context, input_data_for_converter)

    __get_info_from_results(
        initial_context,
        results,
        subdirectories_set,
        subdirectories_to_project_name
    )

    if initial_context.dry:
        return

    sln_cmake = get_cmake_lists(initial_context, initial_context.solution_path, 'r')
    sln_cmake_projects_text = ''
    if sln_cmake is not None:
        sln_cmake_projects_text = sln_cmake.read()
        sln_cmake.close()

    sln_cmake = get_cmake_lists(initial_context, initial_context.solution_path)
    DataConverter.add_cmake_version_required(sln_cmake)
    sln_cmake.write(
        'set(CMAKE_SYSTEM_VERSION {} CACHE TYPE INTERNAL FORCE)\n\n'
        .format(initial_context.target_windows_version)
    )
    sln_cmake.write(
        'project({0})\n\n'.format(os.path.splitext(os.path.basename(sln_path))[0])
    )

    write_arch_types(sln_cmake)

    configuration_types_list = __get_global_configuration_types(solution_data)
    __write_supported_architectures_check(initial_context, sln_cmake)
    __write_global_configuration_types(sln_cmake, configuration_types_list)

    __write_global_compile_options(initial_context, sln_cmake, configuration_types_list)

    __write_global_link_options(sln_cmake, configuration_types_list)

    write_use_package_stub(sln_cmake)

    write_comment(sln_cmake, 'Common utils')
    sln_cmake.write('include(CMake/Utils.cmake)\n\n')
    copy_cmake_utils(initial_context.solution_path)

    write_comment(sln_cmake, 'Additional Global Settings(add specific info there)')
    sln_cmake.write('include(CMake/GlobalSettingsInclude.cmake OPTIONAL)\n\n')

    write_comment(sln_cmake, 'Use solution folders feature')
    sln_cmake.write('set_property(GLOBAL PROPERTY USE_FOLDERS ON)\n\n')

    __write_subdirectories(sln_cmake, subdirectories_set, subdirectories_to_project_name)

    if sln_cmake_projects_text != '':
        sln_cmake.write('\n' * 26)
        sln_cmake.write(sln_cmake_projects_text)

    sln_cmake.close()

    warnings = ''
    if initial_context.warnings_count > 0:
        warnings = ' ({} warnings)'.format(initial_context.warnings_count)
    message(initial_context, 'Conversion of solution finished{}'.format(warnings), 'done')


def __get_input_data_for_converter(initial_context, projects_data):
    input_data_for_converter = {}
    project_number = 0
    projects_filter_pattern = re.compile(initial_context.projects_regexp)
    for guid in projects_data:
        project_number += 1
        project_context = initial_context.clone()
        project_context.project_number = project_number
        project_path = projects_data[guid]['path']

        m = projects_filter_pattern.match(project_path)
        if m is None:
            continue

        project_path = '/'.join(project_path.split('\\'))
        project_abs = os.path.join(initial_context.solution_path, project_path)
        subdirectory = os.path.dirname(project_abs)
        set_dependencies_for_project(project_context, projects_data[guid])
        project_context.sln_configurations_map = \
            projects_data[guid]['sln_configs_2_project_configs']
        project_context.solution_folder = projects_data[guid]['project_solution_dir']
        if subdirectory not in input_data_for_converter:
            input_data_for_converter[subdirectory] = []
        input_data_for_converter[subdirectory].append(
            {
                'project_context': project_context,
                'project_abs': project_abs,
                'subdirectory': subdirectory
            }
        )

    return input_data_for_converter


def __do_conversion(initial_context, input_data_for_converter):
    input_converter_data_list = []
    for subdirectory in input_data_for_converter:
        input_converter_data_list.append(input_data_for_converter[subdirectory])

    results = []
    if initial_context.jobs > 1:
        pool = Pool(initial_context.jobs)
        results = pool.map(run_conversion, input_converter_data_list)
    else:   # do in main thread
        for data_for_converter in input_converter_data_list:
            results.append(run_conversion(data_for_converter))

    return results


def __get_info_from_results(
        initial_context,
        results,
        subdirectories_set,
        subdirectories_to_project_name
):
    for directory_results in results:
        for project_result in directory_results:
            subdirectory = os.path.relpath(project_result['cmake'], initial_context.solution_path)
            if subdirectory != '.':
                subdirectories_set.add(subdirectory)
            subdirectories_to_project_name[subdirectory] = project_result['project_name']
            initial_context.solution_languages.update(project_result['solution_languages'])
            if initial_context.target_windows_version and \
                    project_result['target_windows_ver'] and \
                    initial_context.target_windows_version != project_result['target_windows_ver']:
                message(
                    initial_context,
                    'CMake does not support more than 1 version of windows SDK', 'warn'
                )
            if project_result['target_windows_ver']:
                initial_context.target_windows_version = project_result['target_windows_ver']
            initial_context.warnings_count += project_result['warnings_count']


def __get_global_configuration_types(solution_data):
    configuration_types_set = set()
    for config in solution_data['sln_configurations']:
        configuration_types_set.add(config.split('|')[0])
    configuration_types_list = list(configuration_types_set)
    configuration_types_list.sort(key=str.lower)
    return configuration_types_list


def __write_supported_architectures_check(context, cmake_file):
    arch_list = list(context.supported_architectures)
    arch_list.sort()
    cmake_file.write('if(NOT (')
    first = True
    for arch in arch_list:
        if first:
            cmake_file.write('\"${{CMAKE_VS_PLATFORM_NAME}}\" STREQUAL \"{0}\"'
                             .format(arch))
            first = False
        else:
            cmake_file.write('\n     OR \"${{CMAKE_VS_PLATFORM_NAME}}\" STREQUAL \"{0}\"'
                             .format(arch))
    cmake_file.write('))\n')
    cmake_file.write(
        '    message(FATAL_ERROR "${CMAKE_VS_PLATFORM_NAME} arch is not supported!")\n')
    cmake_file.write('endif()\n\n')


def __write_global_configuration_types(sln_cmake, configuration_types_list):
    write_comment(sln_cmake, 'Global configuration types')
    sln_cmake.write('set(CMAKE_CONFIGURATION_TYPES\n')
    for configuration_type in configuration_types_list:
        sln_cmake.write('    \"{0}\"\n'.format(configuration_type))
    sln_cmake.write('    CACHE STRING "" FORCE\n)\n\n')


def __write_global_compile_options(initial_context, sln_cmake, configuration_types_list):
    write_comment(sln_cmake, 'Global compiler options')
    sln_cmake.write('if(MSVC)\n')
    sln_cmake.write('    # remove default flags provided with CMake for MSVC\n')
    solution_languages = list(initial_context.solution_languages)
    solution_languages.sort(key=str.lower)
    for lang in solution_languages:
        sln_cmake.write('    set(CMAKE_{0}_FLAGS "")\n'.format(lang))
        for configuration_type in configuration_types_list:
            sln_cmake.write('    set(CMAKE_{0}_FLAGS_{1} "")\n'
                            .format(lang, configuration_type.upper()))
    sln_cmake.write('endif()\n\n')


def __write_global_link_options(sln_cmake, configuration_types_list):
    write_comment(sln_cmake, 'Global linker options')
    sln_cmake.write('if(MSVC)\n')
    sln_cmake.write('    # remove default flags provided with CMake for MSVC\n')
    sln_cmake.write('    set(CMAKE_EXE_LINKER_FLAGS "")\n')
    sln_cmake.write('    set(CMAKE_MODULE_LINKER_FLAGS "")\n')
    sln_cmake.write('    set(CMAKE_SHARED_LINKER_FLAGS "")\n')
    sln_cmake.write('    set(CMAKE_STATIC_LINKER_FLAGS "")\n')
    for configuration_type in configuration_types_list:
        ct_upper = configuration_type.upper()
        sln_cmake.write(
            '    set(CMAKE_EXE_LINKER_FLAGS_{0} \"${{CMAKE_EXE_LINKER_FLAGS}}\")\n'
            .format(ct_upper))
        sln_cmake.write(
            '    set(CMAKE_MODULE_LINKER_FLAGS_{0} \"${{CMAKE_MODULE_LINKER_FLAGS}}\")\n'
            .format(ct_upper))
        sln_cmake.write(
            '    set(CMAKE_SHARED_LINKER_FLAGS_{0} \"${{CMAKE_SHARED_LINKER_FLAGS}}\")\n'
            .format(ct_upper))
        sln_cmake.write(
            '    set(CMAKE_STATIC_LINKER_FLAGS_{0} \"${{CMAKE_STATIC_LINKER_FLAGS}}\")\n'
            .format(ct_upper))
    sln_cmake.write('endif()\n\n')


def __write_subdirectories(sln_cmake, subdirectories_set, subdirectories_to_project_name):
    write_comment(sln_cmake, 'Sub-projects')
    subdirectories = list(subdirectories_set)
    subdirectories.sort(key=str.lower)
    for subdirectory in subdirectories:
        binary_dir = ''
        if '.' in subdirectory[:1]:
            binary_dir = ' ${{CMAKE_BINARY_DIR}}/{0}'.format(
                subdirectories_to_project_name[subdirectory])
        sln_cmake.write('add_subdirectory({0}{1})\n'.format(
            set_unix_slash(subdirectory), binary_dir))
    sln_cmake.write('\n')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

import os
import unittest

from cmake_converter.data_converter import DataConverter
from cmake_converter.context import Context


class TestDataConverter(unittest.TestCase):
    """
        This file test methods of DataConverter class.
    """

    cur_dir = os.path.dirname(os.path.realpath(__file__))
    vs_project = '%s/datatest/foo.vcxproj' % cur_dir

    def test_init_files(self):
        """Data Converter Init Files"""

        under_test = Context()

        self.assertEqual(under_test.cmake, '')
        self.assertEqual(under_test.vcxproj, {})

        under_test.init(self.vs_project, self.cur_dir)

        self.assertNotEqual(under_test.cmake, '')
        self.assertIsNotNone(under_test.vcxproj)

        self.assertTrue('ns' in under_test.vcxproj)
        self.assertTrue('tree' in under_test.vcxproj)

    def test_create_data(self):
        """Data Converter Create Data"""

        # FIXME: No such file or directory: 'CMakeLists.txt'
        return

        under_test = DataConverter()

        context = Context()
        context.init(self.vs_project, self.cur_dir)

        old_cmake = open('CMakeLists.txt', 'r')

        under_test.convert(context)

        new_cmake = open('CMakeLists.txt', 'r')

        # Assert content is not the same after
        self.assertEqual(old_cmake.read(), new_cmake.read())

        old_cmake.close()
        new_cmake.close()

    def test_receive_wrong_cmake_path(self):
        """Wrong CMake Path Write in Current Directory"""

        under_test = Context()
        under_test.init(self.vs_project, '/wrong/path/to/cmake')

        # CMakeLists.txt is created in the current directory
        self.assertEqual('CMakeLists.txt', under_test.cmake)

    ''' #TODO: lost feature?
    def test_inclusion_of_cmake_is_written(self):
        """Inclusion of ".cmake" File is Written"""

        data_test = {
            'cmake': './',
            'cmakeoutput': '',
            'vcxproj': self.vs_project,
            'project': self.vs_project,
            'include': '../../test.txt',
            'includecmake': '../../test.cmake',
            'additional': '',
        }
        under_test = DataConverter(data_test)
        under_test.init_files(self.vs_project, '.')
        under_test.data['cmake'] = get_cmake_lists(context, './')

        under_test.create_data()
        under_test.close_cmake_file()

        cmakelists_test = open('CMakeLists.txt')

        content_test = cmakelists_test.read()

        # Includes are added
        self.assertTrue('Include files and directories' in content_test)
        self.assertTrue('../../test.cmake' in content_test)

        # File "test.txt" is not added because it does not exist
        self.assertTrue('../../test.txt' not in content_test)

        cmakelists_test.close()
    '''



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

import os
import unittest

from cmake_converter.context import Context
from cmake_converter.visual_studio.solution import convert_solution


class TestDependencies(unittest.TestCase):
    """
        This file test methods of Dependencies class.
    """

    cur_dir = os.path.dirname(os.path.realpath(__file__))

    def setUp(self):
        context = Context()
        context.verbose = False
        solution_file = '{}/datatest/sln/cpp.sln'.format(self.cur_dir)
        convert_solution(context, os.path.abspath(solution_file))

    def test_write_include_dir(self):
        """Write Include Dirs"""

        with open(self.cur_dir + '/datatest/CMakeLists.txt') as cmake_lists_test:
            cmake_content = cmake_lists_test.read()
            self.assertTrue(
                '${CMAKE_CURRENT_SOURCE_DIR}/external' in cmake_content
            )

    def test_write_dependencies(self):
        """Write Dependencies"""

        with open(self.cur_dir + '/datatest/CMakeLists.txt') as cmake_lists_test:
            content_test = cmake_lists_test.read()
            self.assertTrue('''add_dependencies(${PROJECT_NAME}
    g3log
    zlib
)''' in content_test)
            # self.assertTrue('link_directories(${DEPENDENCIES_DIR}/g3log)' in content_test)

    def test_link_dependencies(self):
        """Link Dependencies"""

        with open(self.cur_dir + '/datatest/CMakeLists.txt') as cmake_lists_test:
            self.assertTrue('''target_link_libraries(${PROJECT_NAME} PUBLIC
    g3log
    zlib
)'''
                            in cmake_lists_test.read())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

"""
    Dependencies
    ============
     Manage directories and libraries of project dependencies
"""

import ntpath
import os

from cmake_converter.data_files import get_vcxproj_data
from cmake_converter.utils import get_global_project_name_from_vcxproj_file, normalize_path, message
from cmake_converter.utils import write_property_of_settings, write_comment
from cmake_converter.utils import is_settings_has_data, replace_vs_vars_with_cmake_vars, \
    resolve_path_variables_of_vs


class Dependencies:
    """
        Class who find and write dependencies of project, additionnal directories...
    """

    @staticmethod
    def write_include_directories(context, cmake_file):
        """
        Write include directories of given context to given CMakeLists.txt file

        :param context: current context data
        :type context: Context
        :param cmake_file: CMakeLists.txt IO wrapper
        :type cmake_file: _io.TextIOWrapper
        """

        has_includes = is_settings_has_data(context.sln_configurations_map,
                                            context.settings,
                                            'inc_dirs')
        if has_includes:
            write_comment(cmake_file, 'Include directories')

        include_directories_specifier = 'PUBLIC'
        if context.private_include_directories:
            include_directories_specifier = 'PRIVATE'

        write_property_of_settings(
            cmake_file, context.settings, context.sln_configurations_map,
            begin_text='target_include_directories(${{PROJECT_NAME}} {}'.format(
                include_directories_specifier
            ),
            end_text=')',
            property_name='inc_dirs',
            separator=';\n',
            in_quotes=True
        )
        if has_includes:
            cmake_file.write('\n')

    @staticmethod
    def set_additional_include_directories(aid_text, setting, context):
        """
        Return additional include directories of given context

        :param aid_text: path to sources
        :type aid_text: str
        :param setting: current setting (Debug|x64, Release|Win32,...)
        :type setting: str
        :param context: current context
        :type context: Context
        :return: include directories of context, separated by semicolons
        :rtype: str
        """

        if not aid_text:
            return

        working_path = os.path.dirname(context.vcxproj_path)
        inc_dir = resolve_path_variables_of_vs(context, aid_text)
        inc_dir = inc_dir.replace('%(AdditionalIncludeDirectories)', '')
        inc_dirs = context.settings[setting]['inc_dirs']
        dirs_raw = []
        for i in inc_dir.split(';'):
            if i:
                dirs_raw.append(i)
                i = normalize_path(context, working_path, i)
                i = replace_vs_vars_with_cmake_vars(context, i)
                inc_dirs.append(i)

        context.settings[setting]['inc_dirs_list'].extend(dirs_raw)

        if inc_dirs:
            message(
                context,
                'Include Directories : {0}'.format(context.settings[setting]['inc_dirs']),
                '')

    @staticmethod
    def get_dependency_target_name(context, vs_project):
        """
        Return dependency target name

        :param context: the context of converter
        :type context: Context
        :param vs_project: path to ".vcxproj" file
        :type vs_project: str
        :return: target name
        :rtype: str
        """

        vcxproj = get_vcxproj_data(context, vs_project)
        project_name = get_global_project_name_from_vcxproj_file(vcxproj)

        if project_name:
            return project_name

        return os.path.splitext(ntpath.basename(vs_project))[0]

    @staticmethod
    def write_target_references(context, cmake_file):
        """
        Write target references on given CMakeLists.txt file

        :param context: current context
        :type context: Context
        :param cmake_file: CMakeLists.txt IO wrapper
        :type cmake_file: _io.TextIOWrapper
        """

        deps_to_write = []
        targets_dependencies_set = set()
        for reference in context.target_references:
            targets_dependencies_set.add(reference)
            deps_to_write.append(reference)
        for sln_dep in context.sln_deps:
            if sln_dep not in targets_dependencies_set:
                targets_dependencies_set.add(sln_dep)
                deps_to_write.append(sln_dep)

        if deps_to_write:
            cmake_file.write('add_dependencies(${PROJECT_NAME}\n')
            for dep in deps_to_write:
                cmake_file.write('    {0}\n'.format(dep))
            cmake_file.write(')\n\n')

    @staticmethod
    def write_link_dependencies(context, cmake_file):
        """
        Write link dependencies of project to given cmake file

        :param context: current context
        :type context: Context
        :param cmake_file: CMakeLists.txt IO wrapper
        :type cmake_file: _io.TextIOWrapper
        """

        if context.target_references:
            cmake_file.write('# Link with other targets.\n')
            cmake_file.write('target_link_libraries(${PROJECT_NAME} PUBLIC\n')
            for reference in context.target_references:
                cmake_file.write('    {}\n'.format(reference))
                msg = 'External library found : {0}'.format(reference)
                message(context, msg, '')
            cmake_file.write(')\n\n')

        if is_settings_has_data(context.sln_configurations_map,
                                context.settings,
                                'add_lib_deps'):
            write_property_of_settings(
                cmake_file, context.settings,
                context.sln_configurations_map,
                begin_text='string(CONCAT ADDITIONAL_LIBRARY_DEPENDENCIES',
                end_text=')',
                property_name='add_lib_deps',
                separator=';\n',
                in_quotes=True,
            )
            cmake_file.write(
                'target_link_libraries(${PROJECT_NAME} PUBLIC '
                '"${ADDITIONAL_LIBRARY_DEPENDENCIES}")\n\n')

        if is_settings_has_data(context.sln_configurations_map,
                                context.settings,
                                'target_link_dirs'):
            write_property_of_settings(
                cmake_file, context.settings,
                context.sln_configurations_map,
                begin_text='target_link_directories(${PROJECT_NAME} PUBLIC',
                end_text=')',
                property_name='target_link_dirs',
                in_quotes=True
            )
            cmake_file.write('\n')

    @staticmethod
    def write_property_sheets(cmake_file, property_indent, config_condition_expr,
                              property_value, width, **kwargs):
        del kwargs
        config = '"${CMAKE_CONFIGURATION_TYPES}" '
        width_diff = 0
        if config_condition_expr is not None:
            config = config_condition_expr.replace('$<CONFIG:', '').replace('>', '')
            config += ' '
            width_diff = len('$<CONFIG:>') - 1

        if property_value:
            for property_sheet_cmake in property_value:
                result_width = width - width_diff
                if result_width < 0:
                    result_width = 0
                cmake_file.write(
                    '{}use_props(${{PROJECT_NAME}} {:<{width}}"{}")\n'.format(
                        property_indent,
                        config,
                        property_sheet_cmake,
                        width=result_width)
                )

    @staticmethod
    def write_target_property_sheets(context, cmake_file):
        """
        Write target property sheets of current context

        :param context: current context
        :type context: Context
        :param cmake_file: CMakeLists.txt IO wrapper
        :type cmake_file: _io.TextIOWrapper
        """

        if is_settings_has_data(context.sln_configurations_map,
                                context.settings,
                                'property_sheets'):
            write_comment(cmake_file, 'Includes for CMake from *.props')
            write_property_of_settings(
                cmake_file, context.settings,
                context.sln_configurations_map,
                begin_text='',
                end_text='',
                property_name='property_sheets',
                write_setting_property_func=Dependencies.write_property_sheets
            )
            cmake_file.write('\n')

    @staticmethod
    def write_target_dependency_packages(context, cmake_file):
        """
        Write target dependency packages of current context

        :param context: current context
        :type context: Context
        :param cmake_file: CMakeLists.txt IO wrapper
        :type cmake_file: _io.TextIOWrapper
        """

        for package in context.packages:
            for package_property in package[2]:
                id_version = '{0}.{1}'.format(package[0], package[1])
                for setting in context.settings:
                    if 'packages' not in context.settings[setting]:
                        continue
                    if id_version not in context.settings[setting]['packages']:
                        continue
                    if package_property not in context.settings[setting]['packages'][id_version]:
                        continue
                    context.settings[setting][id_version + package_property] =\
                        context.settings[setting]['packages'][id_version][package_property]

                package_property_variable = package_property + '_VAR'
                has_written = write_property_of_settings(
                    cmake_file, context.settings,
                    context.sln_configurations_map,
                    begin_text='string(CONCAT "{0}"'.format(package_property_variable),
                    end_text=')',
                    property_name=id_version + package_property,
                )
                if has_written:
                    cmake_file.write(
                        'set_target_properties(${{PROJECT_NAME}} PROPERTIES "{0}" ${{{1}}})\n'
                        .format(package_property, package_property_variable)
                    )
            cmake_file.write(
                'use_package(${{PROJECT_NAME}} {0} {1})\n'.format(package[0], package[1])
            )

    @staticmethod
    def write_target_build_event_of_setting(cmake_file, property_indent, config_condition_expr,
                                            property_value, width, **kwargs):
        del kwargs
        for command in property_value:
            cmake_file.write('{0}    COMMAND {1:>{width}} {2}\n'
                             .format(property_indent, config_condition_expr, command,
                                     width=width))

    @staticmethod
    def __write_target_build_events(context, cmake_file, comment, value_name, event_type):
        has_build_events = is_settings_has_data(context.sln_configurations_map,
                                                context.settings,
                                                value_name)
        if has_build_events:
            write_comment(cmake_file, comment)
            write_property_of_settings(
                cmake_file, context.settings, context.sln_configurations_map,
                begin_text='add_custom_command_if(\n'
                '    TARGET ${{PROJECT_NAME}}\n'
                '    {0}\n'
                '    COMMANDS'.format(event_type),
                end_text=')',
                property_name=value_name,
                write_setting_property_func=Dependencies.write_target_build_event_of_setting
            )
            cmake_file.write('\n')

    @staticmethod
    def write_file_build_event_of_setting(cmake_file, property_indent, config_condition_expr,
                                          property_value, width, **kwargs):
        del kwargs
        # for command in property_value:
        if config_condition_expr is None:
            return
        cmake_file.write('{0}    COMMAND {1:>{width}} {2}\n'
                         .format(property_indent, config_condition_expr,
                                 property_value['command_line'],
                                 width=width))

    @staticmethod
    def __write_file_custom_build_events(context, cmake_file, comment, value_name, event_type):
        del event_type
        has_build_events = False
        for file in context.file_contexts:
            file_context = context.file_contexts[file]
            has_build_events = is_settings_has_data(context.sln_configurations_map,
                                                    file_context.settings,
                                                    value_name)
            if has_build_events:
                break

        if has_build_events:
            write_comment(cmake_file, comment)

            for file in context.file_contexts:
                file_context = context.file_contexts[file]
                outputs = ''
                description = ''
                for setting in file_context.settings:
                    file_setting = file_context.settings[setting]
                    if 'file_custom_build_events' in file_setting:
                        outputs = file_setting['file_custom_build_events']['outputs']
                        description = file_setting['file_custom_build_events']['description']
                text = write_property_of_settings(
                    cmake_file, file_context.settings, context.sln_configurations_map,
                    begin_text='add_custom_command_if(\n'
                    '    OUTPUT "{0}"\n'
                    '    COMMANDS'.format(outputs),
                    end_text='    DEPENDS "{0}"\n'
                    '    COMMENT "{1}"\n)'.format(file, description),
                    property_name='file_custom_build_events',
                    write_setting_property_func=Dependencies.write_file_build_event_of_setting
                )
                if text:
                    cmake_file.write('\n')

    def write_target_pre_build_events(self, context, cmake_file):
        self.__write_target_build_events(
            context,
            cmake_file,
            'Pre build events',
            'pre_build_events',
            'PRE_BUILD'
        )

    def write_target_pre_link_events(self, context, cmake_file):
        self.__write_target_build_events(
            context,
            cmake_file,
            'Pre link events',
            'pre_link_events',
            'PRE_LINK'
        )

    def write_target_post_build_events(self, context, cmake_file):
        self.__write_target_build_events(
            context,
            cmake_file,
            'Post build events',
            'post_build_events',
            'POST_BUILD'
        )

    def write_custom_build_events_of_files(self, context, cmake_file):
        self.__write_file_custom_build_events(
            context,
            cmake_file,
            'Custom build events of files',
            'file_custom_build_events',
            None
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

import os
import re

from cmake_converter.dependencies import Dependencies
from cmake_converter.data_files import get_xml_data, get_propertygroup
from cmake_converter.utils import normalize_path, message, prepare_build_event_cmd_line_for_cmake, \
    check_for_relative_in_path, cleaning_output, set_native_slash, get_mount_point
from cmake_converter.flags import ln_flags


class VCXDependencies(Dependencies):

    def set_include_dirs(self, context, incl_dir):
        """
        Write on "CMakeLists.txt" include directories required for compilation.

        """

        self.set_additional_include_directories(
            incl_dir.text, context.current_setting, context
        )

    def add_target_reference(self, context, attr_name, attr_value, node):
        """
        Find and set references of project to current context

        """

        del attr_name, node

        ref = self.get_dependency_target_name(
            context,
            os.path.join(os.path.dirname(context.vcxproj_path), attr_value)
        )

        context.target_references.append(ref)
        message(context, 'Added reference : {}'.format(ref), '')

    @staticmethod
    def set_target_additional_dependencies(context, dependencies):
        """
        Find and set additional dependencies of current project in context

        """
        list_depends = dependencies.text.replace('%(AdditionalDependencies)', '')
        if list_depends != '':
            add_libs = []
            for d in list_depends.split(';'):
                if d != '%(AdditionalDependencies)':
                    if os.path.splitext(d)[1] == '.lib':
                        add_libs.append(d.replace('.lib', ''))
            message(context, 'Additional Dependencies : {}'.format(add_libs), '')
            context.add_lib_deps = True
            context.settings[context.current_setting]['add_lib_deps'] = add_libs

    @staticmethod
    def set_target_additional_library_directories(context, additional_library_directories):
        """
        Find and set additional library directories in context

        """

        list_depends = additional_library_directories.text.replace(
            '%(AdditionalLibraryDirectories)', ''
        )
        if list_depends != '':
            add_lib_dirs = []
            for d in list_depends.split(';'):
                d = d.strip()
                if d != '':
                    add_lib_dirs.append(
                        check_for_relative_in_path(
                            context,
                            cleaning_output(context, d)
                        )
                    )
            message(context, 'Additional Library Directories = {}'.format(add_lib_dirs), '')
            context.settings[context.current_setting]['target_link_dirs'] = add_lib_dirs

    @staticmethod
    def set_link_library_dependencies(context, node):
        if None in context.current_setting:
            configuration_type = context.settings[context.current_setting]['target_type']
            if node.text.strip() == 'true' and configuration_type == 'StaticLibrary':
                message(
                    context,
                    'LinkLibraryDependencies is true that is not supported with cmake. '
                    'There is a workaround with cmake object libraries but it is hard to '
                    'implement it in converter. '
                    'You should switch LinkLibraryDependencies to false, otherwise you '
                    'might have problems with linking.', 'warn1'
                )

    @staticmethod
    def set_target_ignore_specific_default_libraries(context, node):
        """
        IgnoreSpecificDefaultLibraries node handler

        :param context:
        :param node:
        :return:
        """
        if node.text is None:
            return

        list_ingore_spec_libs = node.text.replace('%(IgnoreSpecificDefaultLibraries)', '')
        if list_ingore_spec_libs != '':
            ignore_libs = []
            for spec_lib in list_ingore_spec_libs.split(';'):
                spec_lib = spec_lib.strip()
                if spec_lib:
                    ignore_libs.append('/NODEFAULTLIB:' + spec_lib)
            message(context, 'Ignore Specific Default Libraries : {}'.format(ignore_libs), '')
            context.settings[context.current_setting][ln_flags] += ignore_libs

    @staticmethod
    def add_target_property_sheet(context, attr_name, filename, node):
        """
        Find and set in current context property sheets

        """

        del attr_name, node

        if 'Microsoft' in filename:  # ignore props provided by Microsoft
            return
        if filename[-8:] == '.targets':  # ignore targets files
            return

        # props_path = os.path.join(os.path.dirname(context.vcxproj_path), filename)
        working_path = os.path.dirname(context.vcxproj_path)
        props_cmake_path = normalize_path(
            context,
            working_path,
            filename,
            False
        ).replace('.props', '.cmake')
        message(context, 'cmake from property sheet: {}'.format(props_cmake_path), '')
        context.settings[context.current_setting]['property_sheets'].append(props_cmake_path)
        # properties_xml = get_xml_data(context, props_path)
        # if properties_xml:
        #     properties_xml.close()  # TODO collect data from props

    @staticmethod
    def __get_info_from_packages_config(context):
        packages_xml = get_xml_data(context, os.path.join(os.path.dirname(context.vcxproj_path),
                                                          context.packages_config_path))
        if not packages_xml:
            return None
        extension = packages_xml['tree'].xpath('/packages/package')
        packages_xml_data = {}
        for ref in extension:
            package_id = ref.get('id')
            package_version = ref.get('version')
            id_version = '{0}.{1}'.format(package_id, package_version)
            packages_xml_data[id_version] = [package_id, package_version]
        return packages_xml_data

    @staticmethod
    def set_target_dependency_packages(context, attr_name, attr_value, ext_targets):
        del attr_name, attr_value
        for import_project_node in ext_targets:
            targets_file_path = import_project_node.get('Project')
            if targets_file_path is None:
                continue

            drive, _ = os.path.splitdrive(targets_file_path)
            if not drive:  # targets_file_path is not absolute
                mount_point = get_mount_point(context.vcxproj_path)
                targets_file_path = set_native_slash(os.path.join(mount_point, targets_file_path))
                targets_file_path = os.path.normpath(targets_file_path)

            context.import_projects.append(targets_file_path)

    def apply_target_dependency_packages(self, context):
        if not context.packages_config_path:
            return

        packages_xml_data = self.__get_info_from_packages_config(context)
        if packages_xml_data is None:
            return

        for targets_file_path in context.import_projects:
            package_id = ''
            package_version = ''
            id_version = ''
            for id_version_i in packages_xml_data:
                if id_version_i in targets_file_path:
                    id_version = id_version_i
                    package_id = packages_xml_data[id_version_i][0]
                    package_version = packages_xml_data[id_version_i][1]
                    break

            if context.import_projects and not id_version:
                message(
                    context,
                    'can not find package version in {} by {} path'.format(
                        context.packages_config_path,
                        targets_file_path,
                    ),
                    'warn'
                )
                continue

            ext_properties = []
            if targets_file_path:
                targets_file = get_xml_data(context, targets_file_path)
                if targets_file:
                    property_page_schema_nodes = targets_file['tree']\
                        .xpath('//ns:ItemGroup/ns:PropertyPageSchema',
                               namespaces=targets_file['ns'])

                    if property_page_schema_nodes:
                        for property_page_schema_node in property_page_schema_nodes:
                            xml_schema_path = property_page_schema_node.get('Include')
                            xml_schema_path = xml_schema_path.replace(
                                '$(MSBuildThisFileDirectory)',
                                os.path.dirname(targets_file_path) + '/'
                            )
                            xml_schema_file = get_xml_data(context, xml_schema_path)
                            if xml_schema_file:
                                ext_property_nodes = xml_schema_file['tree'] \
                                    .xpath('//ns:EnumProperty',
                                           namespaces=xml_schema_file['ns'])
                                for ext_property_node in ext_property_nodes:
                                    ext_properties.append(ext_property_node.get('Name'))
                    # TODO: remove next if due specific hack
                    if not ext_properties:
                        ext_property_nodes = targets_file['tree']\
                            .xpath('//ns:PropertyGroup'
                                   '[@Label="Default initializers for properties"]/*',
                                   namespaces=targets_file['ns'])
                        for ext_property_node in ext_property_nodes:
                            ext_properties.append(re.sub(r'{.*\}', '', ext_property_node.tag))
                else:
                    message(
                        context,
                        "Can't open {} file for package properties searching. Download nupkg "
                        "and rerun converter again".format(targets_file_path), 'warn1')

            context.packages.append([package_id, package_version, ext_properties])
            message(context, 'Used package {0} {1}.'
                    .format(package_id, package_version), '')

            for ext_property in ext_properties:
                for setting in context.settings:
                    if None in setting:
                        continue
                    if 'packages' not in context.settings[setting]:
                        context.settings[setting]['packages'] = {}
                    ext_property_node = context.vcxproj['tree'].xpath(
                        '{0}/ns:{1}'.format(get_propertygroup(setting), ext_property),
                        namespaces=context.vcxproj['ns'])
                    if ext_property_node:
                        if id_version not in context.settings[setting]['packages']:
                            context.settings[setting]['packages'][id_version] = {}
                        context.settings[setting]['packages'][id_version][ext_property] = \
                            [ext_property_node[0].text]
                        message(context, '{0} property of {1} {2} for {3} is {4}'
                                .format(ext_property,
                                        package_id,
                                        package_version,
                                        setting,
                                        ext_property_node[0].text), '')

    @staticmethod
    def __set_target_build_events(context, build_event_node, value_name, event_type):
        for command in build_event_node:
            if 'Command' not in command.tag:
                continue
            if command.text is None:
                continue
            for build_event in command.text.split('\n'):
                build_event = build_event.strip()
                if build_event:
                    cmake_build_event = prepare_build_event_cmd_line_for_cmake(
                        context,
                        build_event
                    )
                    context.settings[context.current_setting][value_name] \
                        .append(cmake_build_event)
                    message(context, '{} event : {}'
                            .format(event_type, cmake_build_event), 'info')

    def set_target_pre_build_events(self, context, node):
        self.__set_target_build_events(
            context,
            node,
            'pre_build_events',
            'Pre build'
        )

    def set_target_pre_link_events(self, context, node):
        self.__set_target_build_events(
            context,
            node,
            'pre_link_events',
            'Pre link'
        )

    def set_target_post_build_events(self, context, node):
        self.__set_target_build_events(
            context,
            node,
            'post_build_events',
            'Post build'
        )

    # TODO: implement
    # def set_custom_build_step(self, context, node):
    #     self.__set_target_build_events(
    #         context,
    #         '{0}/ns:CustomBuildStep/ns:Command',
    #         'custom_build_step',
    #         'Custom build'
    #     )

    def write_target_property_sheets(self, context, cmake_file):
        """
        Write target property sheets of current context

        :param context: current context
        :type context: Context
        :param cmake_file: CMakeLists.txt IO wrapper
        :type cmake_file: _io.TextIOWrapper
        """
        cmake_file.write(
            'use_props(${PROJECT_NAME} "${CMAKE_CONFIGURATION_TYPES}" "${DEFAULT_CXX_PROPS}")\n'
        )
        super(VCXDependencies, self).write_target_property_sheets(context, cmake_file)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

import re
import os

from cmake_converter.project_files import ProjectFiles
from cmake_converter.utils import message


class VFProjectFiles(ProjectFiles):

    def include_directive_case_check(self, context, file_path_name, file_lists_for_include_paths):
        includes_re = re.compile(
            r'include \'(.*)\'', re.IGNORECASE
        )
        working_path = os.path.dirname(context.vcxproj_path)
        file_abs_path = os.path.join(working_path, file_path_name)
        file = open(file_abs_path, 'r', errors='replace')
        file_text = file.read()
        file.close()
        includes = includes_re.findall(file_text)

        checked_includes = set()
        for include_name_in_file in includes:
            if include_name_in_file in checked_includes:
                continue
            checked_includes.add(include_name_in_file)
            include_file_path, include_file_name = os.path.split(include_name_in_file)

            abs_source_path = os.path.dirname(file_abs_path)
            if os.path.exists(abs_source_path):
                file_lists_for_include_paths[abs_source_path] \
                    = os.listdir(abs_source_path)

            found = False
            for include_path in file_lists_for_include_paths:
                files = file_lists_for_include_paths[include_path]
                if include_name_in_file in files:
                    found = True
                    break

            # try to search in relative path
            if not found:
                file_lists_for_relative_include_paths = {}
                for setting in context.settings:
                    for include_path in context.settings[setting]['inc_dirs_list']:
                        if include_path not in file_lists_for_relative_include_paths:
                            abs_include_path = os.path.join(working_path, include_path)
                            abs_include_path_relative = os.path.join(abs_include_path,
                                                                     include_file_path)
                            if os.path.exists(abs_include_path_relative):
                                file_lists_for_relative_include_paths[include_path] = os.listdir(
                                    abs_include_path_relative)
                for include_path in file_lists_for_relative_include_paths:
                    files = file_lists_for_relative_include_paths[include_path]
                    if include_file_name in files:
                        found = True
                        break
            if not found:
                message(context, 'include {0} in file {1} not found'
                        .format(include_name_in_file, file_path_name), 'error')
            file_lists_for_include_paths.pop(abs_source_path)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

from cmake_converter.context_initializer import ContextInitializer
from cmake_converter.data_files import get_xml_data

from .dependencies import VFDependencies
from .flags import FortranFlags
from .project_files import VFProjectFiles
from .project_variables import VFProjectVariables
from .utils import VFUtils
from .parser import VFParser


class VFContextInitializer(ContextInitializer):
    def __init__(self, context, xml_project_path, cmake_lists_destination_path):
        ContextInitializer.__init__(self, context, xml_project_path, cmake_lists_destination_path)
        context.variables = VFProjectVariables()
        context.files = VFProjectFiles()
        context.flags = FortranFlags()
        context.dependencies = VFDependencies()
        context.utils = VFUtils()
        context.parser = VFParser()

    def init_context(self, context, vs_project):
        """
        Initialize context for given VS project

        :param context: converter context
        :type context: Context
        :param vs_project: VS project path (.vcxproj)
        :type vs_project: str
        """

        context.vcxproj = get_xml_data(context, vs_project)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

import time
from multiprocessing import cpu_count
from collections import OrderedDict
import copy

from cmake_converter.visual_studio.vcxproj.context_initializer import VCXContextInitializer
from cmake_converter.visual_studio.vfproj.context_initializer import VFContextInitializer


class Context:
    """
        Converter context
    """
    def __init__(self):
        self.time0 = time.time()
        self.jobs = cpu_count()
        self.vcxproj = {}
        self.vcxproj_path = ''
        self.solution_path = ''
        self.project_number = None
        self.has_headers = False
        self.has_only_headers = False
        self.solution_languages = set()
        self.project_languages = []
        self.sln_deps = []
        self.target_references = []
        self.add_lib_deps = False
        self.packages_config_path = ''
        self.import_projects = []
        self.packages = []

        self.projects_regexp = '.*'
        self.additional_code = None
        self.dry = False
        self.verbose = False
        self.warn_level = 2
        self.private_include_directories = False

        self.sln_configurations_map = dict()
        self.solution_folder = ''
        self.configurations_to_parse = set()
        self.cmake = ''
        self.project_name = ''
        self.root_namespace = ''
        self.target_windows_version = ''
        self.sources = {}
        self.headers = {}
        self.other_project_files = {}
        self.source_groups = {}
        self.file_contexts = OrderedDict()
        self.supported_architectures = set()
        self.settings = OrderedDict()
        self.current_setting = (None, None)  # (conf, arch)
        self.warnings_count = 0
        # helpers
        self.parser = None
        self.variables = None
        self.files = None
        self.flags = None
        self.dependencies = None
        self.utils = None

    def clone(self):
        return copy.deepcopy(self)

    def init(self, xml_project_path, cmake_lists_destination_path):
        context_initializer_map = {
            'vcxproj': VCXContextInitializer,
            'vfproj': VFContextInitializer
        }

        for key in context_initializer_map:
            if key in xml_project_path:
                context_initializer_map[key](self, xml_project_path, cmake_lists_destination_path)
                self.utils.init_context_current_setting(self)  # None - global settings
                self.flags.prepare_context_for_flags(self)
                return True
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Pavel Liavonau, liavonlida@gmail.com
#   Matthieu Estrada, ttamalfor@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

import re

from cmake_converter.utils import message


class StopParseException(Exception):
    pass


class Parser:

    def __init__(self):
        self.reset_setting_after_nodes = set()

    @staticmethod
    def parse(context):
        message(context, 'Parser is not implemented yet!', 'error')

    @staticmethod
    def do_nothing_node_stub(context, node):
        pass

    @staticmethod
    def do_nothing_attr_stub(context, attr_name, param, node):
        pass

    def get_node_handlers_dict(self, context):
        raise NotImplementedError('You need to define a get_node_handlers_dict method!')

    def get_attribute_handlers_dict(self, context):
        raise NotImplementedError('You need to define a get_attribute_handlers_dict method!')

    def reset_current_setting_after_parsing_node(self, node):
        self.reset_setting_after_nodes.add(node)

    @staticmethod
    def strip_namespace(tag):
        return re.sub(r'{.*\}', '', tag)

    def _parse_nodes(self, context, parent):
        for child_node in parent:
            if not isinstance(child_node.tag, str):
                continue
            child_node_tag = Parser.strip_namespace(child_node.tag)

            try:
                self._parse_attributes(context, child_node)
            except StopParseException:
                continue

            node_handlers = self.get_node_handlers_dict(context)
            if child_node_tag in node_handlers:
                node_handlers[child_node_tag](context, child_node)
            else:
                message(context, 'No handler for <{}> node.'.format(child_node_tag), 'warn3')

            if child_node in self.reset_setting_after_nodes:
                context.current_setting = (None, None)
                self.reset_setting_after_nodes.remove(child_node)

    def _parse_attributes(self, context, node):
        for attr in node.attrib:
            node_tag = Parser.strip_namespace(node.tag)
            node_key = '{}_{}'.format(node_tag, attr)
            attributes_handlers = self.get_attribute_handlers_dict(context)
            if node_key in attributes_handlers:    # node specified handler
                attributes_handlers[node_key](context, node_key, node.get(attr), node)
            elif attr in attributes_handlers:      # common attribute handler
                attributes_handlers[attr](context, attr, node.get(attr), node)
            else:
                message(
                    context,
                    'No handler for "{}" attribute of <{}> node.'.format(attr, node_tag),
                    'warn3'
                )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Pavel Liavonau, liavonlida@gmail.com
#   Matthieu Estrada, ttamalfor@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

import re

from cmake_converter.parser import Parser, StopParseException
from cmake_converter.data_files import get_xml_data
from cmake_converter.utils import get_actual_filename


class VCXParser(Parser):

    def __init__(self):
        Parser.__init__(self)
        self.filters = None

    def get_node_handlers_dict(self, context):
        node_handlers = {}

        node_handlers.update(
            dict.fromkeys(context.flags.flags_handlers.keys(), context.flags.set_flag)
        )

        node_handlers.update({
            'ItemGroup': self.__parse_item_group,
            'Target': self._parse_nodes,
            'ProjectConfiguration': self.do_nothing_node_stub,
            'ConfigurationType': self.__parse_configuration_type,
            'CharacterSet': context.flags.set_character_set,
            'PlatformToolset': self.do_nothing_node_stub,
            'PropertyGroup': self.__parse_property_group,
            'ProjectName': self.do_nothing_node_stub,  # handled globally, ignore here
            'ProjectGuid': self.do_nothing_node_stub,
            'RootNamespace': context.variables.set_root_namespace,
            'Keyword': context.variables.set_keyword,
            'RestorePackages': self.do_nothing_node_stub,             # no support in CMake
            '_ProjectFileVersion': self.do_nothing_node_stub,
            'VCProjectVersion': self.do_nothing_node_stub,
            'CodeAnalysisRuleSet': self.do_nothing_node_stub,         # no support in CMake
            'CodeAnalysisRules': self.do_nothing_node_stub,           # no support in CMake
            'CodeAnalysisRuleAssemblies': self.do_nothing_node_stub,  # no support in CMake
            'RunCodeAnalysis': self.do_nothing_node_stub,             # no support in CMake
            'SccProjectName': self.do_nothing_node_stub,              # no support in CMake
            'SccAuxPath': self.do_nothing_node_stub,                  # no support in CMake
            'SccLocalPath': self.do_nothing_node_stub,                # no support in CMake
            'SccProvider': self.do_nothing_node_stub,                 # no support in CMake
            'WindowsTargetPlatformVersion': context.variables.set_windows_target_version,
            'ItemDefinitionGroup': self.__parse_item_definition_group,
            'Import': self._parse_nodes,
            'ClInclude': self._parse_nodes,
            'ClCompile': self._parse_nodes,
            'None': self._parse_nodes,
            'Text': self._parse_nodes,
            'SubType': self.do_nothing_node_stub,  # no support in CMake
            'PreBuildEvent': context.dependencies.set_target_pre_build_events,
            'PreLinkEvent': context.dependencies.set_target_pre_link_events,
            'PostBuildEvent': context.dependencies.set_target_post_build_events,
            'ProjectReference': self._parse_nodes,
            'Project': self.do_nothing_node_stub,  # just GUID at <ProjectReference>
            'Private': self.do_nothing_node_stub,                       # no support in CMake
            'ReferenceOutputAssembly': self.do_nothing_node_stub,       # no support in CMake
            'CopyLocalSatelliteAssemblies': self.do_nothing_node_stub,  # no support in CMake
            'LinkLibraryDependencies': context.dependencies.set_link_library_dependencies,
            'UseLibraryDependencyInputs': self.do_nothing_node_stub,    # no support in CMake
            'AdditionalIncludeDirectories': context.dependencies.set_include_dirs,
            'AdditionalDependencies': context.dependencies.set_target_additional_dependencies,
            'AdditionalLibraryDirectories':
                context.dependencies.set_target_additional_library_directories,
            'IgnoreSpecificDefaultLibraries':
                context.dependencies.set_target_ignore_specific_default_libraries,
            'PreprocessorDefinitions': context.flags.set_defines,
            'PrecompiledHeaderOutputFile': self.do_nothing_node_stub,  # no GenEx at OBJECT_OUTPUTS
            'Link': self._parse_nodes,
            'Lib': self._parse_nodes,
            'ImportGroup': self._parse_nodes,
            'ProjectExtensions': self.do_nothing_node_stub,  # no support in CMake
            'OutputFile': context.variables.set_output_file,
            'ImportLibrary': context.variables.set_import_library,
            'ProgramDatabaseFile': context.variables.set_program_database_file,
            'ProgramDataBaseFileName': self.do_nothing_node_stub,  # no support of GenEx in CMake
            'IntDir': self.do_nothing_node_stub,  # no analog at CMake
            'OutDir': context.variables.set_output_dir,
            'TargetName': self.__parse_target_name_node,
            'EnablePREfast': self.do_nothing_node_stub,     # no support from CMake
            'AdditionalOptions': self.__parse_additional_options,
        })
        return node_handlers

    def get_attribute_handlers_dict(self, context):
        attributes_handlers = {
            'ItemGroup_Label': self.do_nothing_attr_stub,
            'PropertyGroup_Label': self.do_nothing_attr_stub,
            'Import_Label': self.do_nothing_attr_stub,
            'Import_Project': context.dependencies.add_target_property_sheet,
            'ImportGroup_Label': self.__parse_import_group_label,
            'ProjectConfiguration_Include': self.__parse_project_configuration_include,
            'ClCompile_Include': self.__parse_cl_compile_include_attr,
            'ClInclude_Include': self.__parse_cl_include_include_attr,
            'None_Include': self.__parse_cl_none_include_attr,
            'Text_Include': self.__parse_cl_none_include_attr,
            'Condition': self.__parse_condition,
            'ProjectReference_Include': context.dependencies.add_target_reference,
            'Target_Name': self.__parse_target_name_attr,
        }
        return attributes_handlers

    def parse(self, context):
        filters_file = get_actual_filename(context, context.vcxproj_path + '.filters')
        if filters_file is not None:
            self.filters = get_xml_data(context, filters_file)
        tree = context.vcxproj['tree']
        root = tree.getroot()
        self._parse_nodes(context, root)
        context.flags.apply_flags_to_context(context)
        context.files.apply_files_to_context(context)
        context.dependencies.apply_target_dependency_packages(context)
        if not context.has_only_headers:
            context.flags.define_pch_cpp_file(context)

    def __parse_item_group(self, context, node):
        self._parse_nodes(context, node)

    @staticmethod
    def __parse_import_group_label(context, attr_name, attr_value, node):
        if attr_value in ['ExtensionTargets', 'Shared', 'ExtensionSettings']:
            context.dependencies.set_target_dependency_packages(
                context,
                attr_name,
                attr_value,
                node
            )

    @staticmethod
    def __parse_project_configuration_include(context, attr_name, setting,
                                              project_configuration_node):
        del attr_name, project_configuration_node

        setting = tuple(setting.split('|'))

        if setting not in context.configurations_to_parse:
            return

        context.current_setting = setting
        context.utils.init_context_current_setting(context)

        context.current_setting = (None, None)

    @staticmethod
    def __parse_configuration_type(context, node):
        context.settings[context.current_setting]['target_type'] = node.text

    def __parse_item_definition_group(self, context, node):
        self._parse_nodes(context, node)

    def __parse_property_group(self, context, node):
        self._parse_nodes(context, node)

    def __get_source_group_from_filters(self, node, filter_node_name):
        if self.filters:
            file_path = node.attrib['Include']
            filter_node = self.filters['tree'].xpath(
                '//ns:{}[@Include="{}"]/*'.format(filter_node_name, file_path),
                namespaces=self.filters['ns']
            )
            if filter_node:
                return filter_node[0].text.replace('\\', '\\\\')

        return ''

    def __parse_cl_include_include_attr(self, context, attr_name, value, include_node):
        del attr_name, value

        self.__parse_file_nodes(context, context.headers, include_node, 'Headers')
        raise StopParseException()

    def __parse_cl_compile_include_attr(self, context, attr_name, value, cl_node):
        del attr_name, value

        self.__parse_file_nodes(context, context.sources, cl_node, 'Sources')
        raise StopParseException()

    def __parse_cl_none_include_attr(self, context, attr_name, value, none_node):
        del attr_name, value

        if 'packages.config' in none_node.attrib['Include']:
            context.packages_config_path = none_node.attrib['Include']
        else:
            self.__parse_file_nodes(context, context.other_project_files, none_node, '')

        raise StopParseException()

    def __parse_file_nodes(self, context, files_container, file_node, source_group):
        if self.filters:
            source_group = self.__get_source_group_from_filters(
                file_node,
                Parser.strip_namespace(file_node.tag)
            )

        file_context = context.files.add_file_from_node(
            context,
            files_container=files_container,
            file_node=file_node,
            file_node_attr='Include',
            source_group=source_group
        )
        self._parse_nodes(file_context, file_node)
        context.warnings_count += file_context.warnings_count

    @staticmethod
    def __parse_additional_options(context, node):
        parent = node.getparent()
        parent_tag = Parser.strip_namespace(parent.tag)
        if parent_tag == 'ClCompile':
            node.tag = 'CompileAdditionalOptions'
            context.flags.set_flag(context, node)
        if parent_tag == 'Link':
            node.tag = 'LinkAdditionalOptions'
            context.flags.set_flag(context, node)

    @staticmethod
    def __parse_target_name_node(context, node):
        context.variables.set_target_name(context, node.text)

    def __parse_condition(self, context, attr_name, condition_value, node):
        del attr_name

        found = re.search(r".*=='(.*)'", condition_value)
        if not found:
            return
        setting = tuple(found.group(1).split('|'))
        if setting in context.settings:
            context.current_setting = setting
            context.flags.prepare_context_for_flags(context)
            self.reset_current_setting_after_parsing_node(node)
        else:
            context.current_setting = (None, None)
            raise StopParseException()

    @staticmethod
    def __parse_target_name_attr(context, attr_name, target_name_value, node):
        del context, attr_name, node

        if target_name_value == 'EnsureNuGetPackageBuildImports':
            raise StopParseException()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

from cmake_converter.utils import Utils


class VCXUtils(Utils):

    def lists_of_settings_to_merge(self):
        lists = super(VCXUtils, self).lists_of_settings_to_merge()
        lists.extend(
            [
                'cl_flags',
                'ln_flags'
            ]
        )
        return lists

    def init_context_current_setting(self, context):
        """
        Define settings of converter.

        :param context: converter context
        :type context: Context
        """

        super(VCXUtils, self).init_context_current_setting(context)
        for l in self.lists_of_settings_to_merge():
            context.settings[context.current_setting][l] = []
        context.settings[context.current_setting]['inc_dirs_list'] = []
        context.settings[context.current_setting]['PrecompiledHeader'] = []
        context.settings[context.current_setting]['property_sheets'] = []



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

import os
import unittest

from cmake_converter.context import Context
from cmake_converter.visual_studio.solution import convert_solution
from cmake_converter.project_files import ProjectFiles
from cmake_converter.data_files import get_cmake_lists


class TestProjectFiles(unittest.TestCase):
    """
        This file test methods of ProjectFiles class.
    """

    context = Context()
    cur_dir = os.path.dirname(os.path.realpath(__file__))

    def setUp(self):
        self.context.verbose = False
        solution_file = '{}/datatest/sln/cpp.sln'.format(self.cur_dir)
        convert_solution(self.context, os.path.abspath(solution_file))

    @unittest.skip("how to test sources from project context?")
    def test_collects_source_files(self):
        """Collects Source Files"""

        self.assertNotEqual(len(self.context.sources), 0)
        self.assertEqual(len(self.context.headers), 0)

    def test_write_source_files(self):
        """Write Source Files"""

        with open('{}/datatest/CMakeLists.txt'.format(self.cur_dir)) as cmake_lists_test:
            content_test = cmake_lists_test.read()
            self.assertTrue('source_group("Sources" FILES ${Sources})' in content_test)

    @unittest.skip("repair test additional code")
    def test_add_additional_code(self):
        """Add Additional CMake Code"""

        # When file is empty, nothing is added
        self.data_test['cmake'] = get_cmake_lists(context, self.cur_dir)
        under_test = ProjectFiles(self.data_test)

        under_test.add_additional_code(context, '')

        under_test.cmake.close()

        cmakelists_test = open('%s/CMakeLists.txt' % self.cur_dir)
        content_test = cmakelists_test.read()

        self.assertEqual('', content_test)
        cmakelists_test.close()

        # When file exist, code is added
        under_test.cmake = get_cmake_lists(context, self.cur_dir)
        under_test.add_additional_code(context, '%s/datatest/additional_code_test.cmake' % self.cur_dir)

        under_test.cmake.close()

        cmakelists_test = open('%s/CMakeLists.txt' % self.cur_dir)
        content_test = cmakelists_test.read()

        self.assertTrue('set(ADD_CODE code)' in content_test)

        cmakelists_test.close()

        # When file does not exist, nothing is added
        under_test.cmake = get_cmake_lists(context, self.cur_dir)
        under_test.add_additional_code(context, 'nofile/additional_code_test.cmake')

        under_test.cmake.close()

        cmakelists_add_test = open('%s/CMakeLists.txt' % self.cur_dir)
        content_add_test = cmakelists_add_test.read()

        self.assertEqual('', content_add_test)

        cmakelists_test.close()

    def test_add_artefacts(self):
        """Add Artefact Target"""

        with open('{}/datatest/CMakeLists.txt'.format(self.cur_dir)) as cmake_lists_test:
            content_test = cmake_lists_test.read()
            self.assertTrue('add_executable(${PROJECT_NAME} ${ALL_FILES})' in content_test)

    @unittest.skip("include_cmake deleted")
    def test_add_include_cmake(self):
        """Add Include CMake File"""

        self.data_test['cmake'] = get_cmake_lists(context, self.cur_dir)
        under_test = ProjectFiles(self.data_test)

        under_test.add_include_cmake('path/to/file.cmake')
        self.data_test['cmake'].close()

        cmakelists_test = open('%s/CMakeLists.txt' % self.cur_dir)
        content_test = cmakelists_test.read()

        self.assertTrue('include("path/to/file.cmake")' in content_test)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016-2018:
#   Matthieu Estrada, ttamalfor@gmail.com
#   Pavel Liavonau, liavonlida@gmail.com
#
# This file is part of (CMakeConverter).
#
# (CMakeConverter) is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# (CMakeConverter) is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with (CMakeConverter).  If not, see <http://www.gnu.org/licenses/>.

"""
    DataConverter
    =============
     Manage conversion of **.vcxproj** data
"""

import os
from collections import OrderedDict

from cmake_converter.data_files import get_cmake_lists
from cmake_converter.flags import Flags
from cmake_converter.utils import write_comment, message, get_mapped_arch
from cmake_converter.context import Context


class DataConverter:
    """
        Base class for converters
    """

    @staticmethod
    def collect_data(context):
        """
        Collect data for converter.

        """

        context.parser.parse(context)

        context.files.find_cmake_project_languages(context)

    @staticmethod
    def verify_data(context):
        target_types = set()
        for setting in context.settings:
            if None in setting:
                continue
            target_types.add(context.settings[setting]['target_type'])

        if len(target_types) > 1:
            message(
                context,
                'Target has more than one output binary type. CMake does not support it!',
                'warn'
            )

    @staticmethod
    def merge_data_settings(context):
        """

        :param context:
        :return:
        """
        for arch in context.supported_architectures:
            mapped_arch = get_mapped_arch(context.sln_configurations_map, arch)
            context.settings[(None, mapped_arch)] = {}
            context.current_setting = (None, mapped_arch)
            context.utils.init_context_current_setting(context)
            merged_settings = context.settings[(None, mapped_arch)]

            for key in context.utils.lists_of_settings_to_merge():
                lists_of_items_to_merge = OrderedDict()
                set_of_items = set()

                # get intersection pass
                for setting in context.settings:
                    if key not in context.settings[setting] \
                            or setting[1] != mapped_arch \
                            or setting[0] is None:
                        continue
                    settings_list = context.settings[setting][key]
                    if not lists_of_items_to_merge:  # first pass
                        set_of_items = set(settings_list)

                    lists_of_items_to_merge[setting] = settings_list
                    set_of_items = set_of_items.intersection(set(context.settings[setting][key]))

                # removing common settings from configurations
                for setting in lists_of_items_to_merge:
                    settings_list = lists_of_items_to_merge[setting]
                    result_settings_list = []
                    for element in settings_list:
                        if element not in set_of_items:
                            result_settings_list.append(element)
                    context.settings[setting][key] = result_settings_list

                merged_order_list = DataConverter.__get_order_of_common_settings(
                    lists_of_items_to_merge
                )

                # getting ordered common settings
                common_ordered_list = []
                for element in merged_order_list:
                    if element in set_of_items:
                        common_ordered_list.append(element)
                        set_of_items.remove(element)
                        if not set_of_items:
                            break

                merged_settings[key] = common_ordered_list

            context.sln_configurations_map[(None, arch)] = (None, mapped_arch)

        if context.file_contexts is not None:
            for file in context.file_contexts:
                DataConverter.merge_data_settings(context.file_contexts[file])

    @staticmethod
    def __get_order_of_common_settings(lists_of_items_to_merge):
        merged_order_list = []
        i = 0
        while True:
            out_of_bounds = 0
            for setting in lists_of_items_to_merge:
                settings_list = lists_of_items_to_merge[setting]
                if i < len(settings_list):
                    merged_order_list.append(settings_list[i])
                else:
                    out_of_bounds += 1

            if out_of_bounds == len(lists_of_items_to_merge):
                break
            i += 1
        return merged_order_list

    @staticmethod
    def write_data(context, cmake_file):
        """
        Write data defined in converter.

        :param context: converter context
        :type context: Context
        :param cmake_file: CMakeLists IO wrapper
        :type cmake_file: _io.TextIOWrapper
        """

        context.files.write_cmake_project(context, cmake_file)

        # Add additional code or not
        if context.additional_code is not None:
            context.files.add_additional_code(context, context.additional_code, cmake_file)

        context.files.write_source_groups(context, cmake_file)

        if not context.has_only_headers:
            write_comment(cmake_file, 'Target')
            context.flags.write_target_artifact(context, cmake_file)
            context.flags.write_use_pch_function(context, cmake_file)
            context.dependencies.write_target_property_sheets(context, cmake_file)
            context.variables.write_target_outputs(context, cmake_file)
            context.dependencies.write_include_directories(context, cmake_file)
            context.flags.write_defines(context, cmake_file)
            context.flags.write_flags(context, cmake_file)
            context.dependencies.write_target_pre_build_events(context, cmake_file)
            context.dependencies.write_target_pre_link_events(context, cmake_file)
            context.dependencies.write_target_post_build_events(context, cmake_file)
            context.dependencies.write_custom_build_events_of_files(context, cmake_file)
            if (context.target_references
                    or context.add_lib_deps
                    or context.sln_deps
                    or context.packages):
                write_comment(cmake_file, 'Dependencies')
            context.dependencies.write_target_references(context, cmake_file)
            context.dependencies.write_link_dependencies(context, cmake_file)
            context.dependencies.write_target_dependency_packages(context, cmake_file)
        else:
            Flags.write_target_headers_only_artifact(context, cmake_file)

    def convert(self, context):
        """
        Method template for data collecting and writing

        """
        message(context, 'Conversion started: Project {0}'.format(context.project_name), 'done')
        message(context, 'Collecting data for project {0}'.format(context.vcxproj_path), '')
        self.collect_data(context)
        self.verify_data(context)
        self.merge_data_settings(context)
        if context.dry:
            return
        if os.path.exists(context.cmake + '/CMakeLists.txt'):
            cmake_file = get_cmake_lists(context, context.cmake, 'a')
            cmake_file.write('\n' * 26)
        else:
            cmake_file = get_cmake_lists(context, context.cmake)
        message(context, 'Writing data for project {0}'.format(context.vcxproj_path), '')
        self.write_data(context, cmake_file)
        cmake_file.close()
        warnings = ''
        if context.warnings_count > 0:
            warnings = ' ({} warnings)'.format(context.warnings_count)
        message(
            context,
            'Conversion done   : Project {}{}'.format(context.project_name, warnings), 'done'
        )

    @staticmethod
    def add_cmake_version_required(cmake_file):
        """
        Write CMake minimum required

        :param cmake_file: IO cmake file wrapper
        :type cmake_file: _io.TextIOWrapper
        """

        cmake_file.write('cmake_minimum_required(VERSION 3.13.0 FATAL_ERROR)\n\n')
